import unittest
import sys
from pythran.tests import TestEnv
import numpy

from pythran.typing import NDArray, List


@TestEnv.module
class TestNumpyFunc1(TestEnv):

    def test_sum_bool2(self):
        self.run_test("def np_sum_bool2(a): return a.sum()", numpy.ones(10,dtype=bool).reshape(2,5), np_sum_bool2=[NDArray[bool,:,:]])

    def test_sum_expr(self):
        self.run_test("def np_sum_expr(a):\n from numpy import ones\n return (a + ones(10)).sum()", numpy.arange(10), np_sum_expr=[NDArray[int,:]])

    def test_sum2_(self):
        self.run_test("def np_sum2_(a): return a.sum()", numpy.arange(10).reshape(2,5), np_sum2_=[NDArray[int,:,:]])

    def test_sum3_(self):
        self.run_test("def np_sum3_(a): return a.sum(1)", numpy.arange(10).reshape(2,5), np_sum3_=[NDArray[int,:,:]])

    def test_sum4_(self):
        self.run_test("def np_sum4_(a): return a.sum(0)", numpy.arange(10).reshape(2,5), np_sum4_=[NDArray[int,:,:]])

    def test_sum5_(self):
        self.run_test("def np_sum5_(a): return a.sum(0)", numpy.arange(10), np_sum5_=[NDArray[int,:]])

    def test_sum6_(self):
        self.run_test("def np_sum6_(a): return a.sum(0)", numpy.arange(12).reshape(2,3,2), np_sum6_=[NDArray[int,:,:,:]])

    def test_sum7_(self):
        self.run_test("def np_sum7_(a): return a.sum(1)", numpy.arange(12).reshape(2,3,2), np_sum7_=[NDArray[int,:,:,:]])

    def test_sum8_(self):
        self.run_test("def np_sum8_(a): return a.sum(2)", numpy.arange(12).reshape(2,3,2), np_sum8_=[NDArray[int,:,:,:]])

    def test_sum9_(self):
        self.run_test("def np_sum9_(a): import numpy as np ; return np.sum(a*a,0)", numpy.arange(12).reshape(2,3,2), np_sum9_=[NDArray[int,:,:,:]])

    def test_sum10_(self):
        self.run_test("def np_sum10_(a): import numpy as np ; return np.sum(a-a,1)", numpy.arange(12).reshape(2,3,2), np_sum10_=[NDArray[int,:,:,:]])

    def test_sum11_(self):
        self.run_test("def np_sum11_(a): import numpy as np ; return np.sum(a+a,2)", numpy.arange(12).reshape(2,3,2), np_sum11_=[NDArray[int,:,:,:]])

    @unittest.skipIf(sys.maxsize == (2**31 - 1), "overflow test")
    def test_sum12_(self):
        self.run_test("def np_sum12_(a): import numpy as np ; return np.sum(a)",
                      numpy.array([2**32-1, -2**32 +1 , -2**32 + 1], dtype=numpy.uint32), np_sum12_=[NDArray[numpy.uint32,:]])

    def test_sum13_(self):
        self.run_test("def np_sum13_(a): import numpy as np ; return np.sum(a)",
                      numpy.array([2**31-1, -2**31 +1 , -2**31 + 1], dtype=numpy.int32), np_sum13_=[NDArray[numpy.int32,:]])

    @unittest.skipIf(sys.maxsize == (2**31 - 1), "overflow test")
    def test_sum14_(self):
        self.run_test("def np_sum14_(a): import numpy as np ; return np.sum(a)",
                      numpy.array([2**31-1, 2**31 +1 , 2**31 + 1], dtype=numpy.int32), np_sum14_=[NDArray[numpy.int32,:]])

    def test_sum15_(self):
        self.run_test("def np_sum15_(a): import numpy as np ; return np.sum(a, dtype=int)",
                      numpy.array([0.5, 1.5, 2.5]), np_sum15_=[NDArray[float,:]])

    def test_sum16_(self):
        self.run_test("def np_sum16_(a): import numpy as np ; return np.sum(a, dtype=int, axis=0)",
                      numpy.array([0.5, 1.5, 2.5]), np_sum16_=[NDArray[float,:]])

    def test_prod_(self):
        """ Check prod function for numpy array. """
        self.run_test("""
            def np_prod_(a):
                return a.prod()""",
                      numpy.arange(10),
                      np_prod_=[NDArray[int,:]])

    def test_prod_bool(self):
        self.run_test("def np_prod_bool(a): return (a > 2).prod()", numpy.arange(10), np_prod_bool=[NDArray[int,:]])

    def test_prod_bool2(self):
        self.run_test("def np_prod_bool2(a): return a.prod()", numpy.ones(10,dtype=bool).reshape(2,5), np_prod_bool2=[NDArray[bool,:,:]])

    def test_prod2_(self):
        self.run_test("def np_prod2_(a): return a.prod()", numpy.arange(10).reshape(2,5), np_prod2_=[NDArray[int,:,:]])

    def test_prod3_(self):
        self.run_test("def np_prod3_(a): return a.prod(1)", numpy.arange(10).reshape(2,5), np_prod3_=[NDArray[int,:,:]])

    def test_prod4_(self):
        self.run_test("def np_prod4_(a): return a.prod(0)", numpy.arange(10).reshape(2,5), np_prod4_=[NDArray[int,:,:]])

    def test_prod5_(self):
        self.run_test("def np_prod5_(a): return a.prod(0)", numpy.arange(10), np_prod5_=[NDArray[int,:]])

    def test_prod6_(self):
        self.run_test("def np_prod6_(a): return a.prod(0)", numpy.arange(12).reshape(2,3,2), np_prod6_=[NDArray[int,:,:,:]])

    def test_prod7_(self):
        self.run_test("def np_prod7_(a): return a.prod(1)", numpy.arange(12).reshape(2,3,2), np_prod7_=[NDArray[int,:,:,:]])

    def test_prod8_(self):
        self.run_test("def np_prod8_(a): return a.prod(2)", numpy.arange(12).reshape(2,3,2), np_prod8_=[NDArray[int,:,:,:]])

    def test_prod9_(self):
        self.run_test("def np_prod9_(a): import numpy as np ; return np.prod(a*a,0)", numpy.arange(12).reshape(2,3,2), np_prod9_=[NDArray[int,:,:,:]])

    def test_prod10_(self):
        self.run_test("def np_prod10_(a): import numpy as np ; return np.prod(a-a,1)", numpy.arange(12).reshape(2,3,2), np_prod10_=[NDArray[int,:,:,:]])

    def test_prod11_(self):
        self.run_test("def np_prod11_(a): import numpy as np ; return np.prod(a+a,2)", numpy.arange(12).reshape(2,3,2), np_prod11_=[NDArray[int,:,:,:]])

    def test_prod_expr(self):
        self.run_test("def np_prod_expr(a):\n from numpy import ones\n return (a + ones(10)).prod()", numpy.arange(10), np_prod_expr=[NDArray[int,:]])

    def test_amin_amax(self):
        self.run_test("def np_amin_amax(a):\n from numpy import amin,amax\n return amin(a), amax(a)",numpy.arange(10),  np_amin_amax=[NDArray[int,:]])

    def test_min_(self):
        self.run_test("def np_min_(a): return a.min()", numpy.arange(10), np_min_=[NDArray[int,:]])

    def test_min1_(self):
        self.run_test("def np_min1_(a): return (a+a).min()", numpy.arange(10), np_min1_=[NDArray[int,:]])

    def test_min2_(self):
        self.run_test("def np_min2_(a): return a.min()", numpy.arange(10).reshape(2,5), np_min2_=[NDArray[int,:,:]])

    def test_min3_(self):
        self.run_test("def np_min3_(a): return a.min(1)", numpy.arange(10).reshape(2,5), np_min3_=[NDArray[int,:,:]])

    def test_min4_(self):
        self.run_test("def np_min4_(a): return a.min(0)", numpy.arange(10).reshape(2,5), np_min4_=[NDArray[int,:,:]])

    def test_min5_(self):
        self.run_test("def np_min5_(a): return a.min(0)", numpy.arange(10), np_min5_=[NDArray[int,:]])

    def test_min6_(self):
        self.run_test("def np_min6_(a): return a.min(1)", numpy.arange(30).reshape(2,5,3), np_min6_=[NDArray[int,:,:,:]])

    def test_min7_(self):
        self.run_test("def np_min7_(a): return (a+a).min(1)", numpy.arange(30).reshape(2,5,3), np_min7_=[NDArray[int,:,:,:]])

    def test_min8_(self):
        self.run_test("def np_min8_(a): return a.min()", numpy.arange(4, dtype=numpy.int8), np_min8_=[NDArray[numpy.int8,:]])

    def test_max_(self):
        self.run_test("def np_max_(a): return a.max()", numpy.arange(10), np_max_=[NDArray[int,:]])

    def test_max1_(self):
        self.run_test("def np_max1_(a): return (a+a).max()", numpy.arange(10), np_max1_=[NDArray[int,:]])

    def test_max2_(self):
        self.run_test("def np_max2_(a): return a.max()", numpy.arange(10).reshape(2,5), np_max2_=[NDArray[int,:,:]])

    def test_max3_(self):
        self.run_test("def np_max3_(a): return a.max(1)", numpy.arange(10).reshape(2,5), np_max3_=[NDArray[int,:,:]])

    def test_max4_(self):
        self.run_test("def np_max4_(a): return a.max(0)", numpy.arange(10).reshape(2,5), np_max4_=[NDArray[int,:,:]])

    def test_max5_(self):
        self.run_test("def np_max5_(a): return a.max(0)", numpy.arange(10), np_max5_=[NDArray[int,:]])

    def test_max6_(self):
        self.run_test("def np_max6_(a): return a.max(1)", numpy.arange(30).reshape(2,5,3), np_max6_=[NDArray[int,:,:,:]])

    def test_max7_(self):
        self.run_test("def np_max7_(a): return (a+a).max(1)", numpy.arange(30).reshape(2,5,3), np_max7_=[NDArray[int,:,:,:]])

    def test_max8_(self):
        self.run_test("def np_max8_(a): return a.max()", numpy.arange(4, dtype=numpy.int8), np_max8_=[NDArray[numpy.int8,:]])

    def test_all_(self):
        self.run_test("def np_all_(a): return a.all()", numpy.arange(10), np_all_=[NDArray[int,:]])

    def test_all2_(self):
        self.run_test("def np_all2_(a): return a.all()", numpy.ones(10).reshape(2,5), np_all2_=[NDArray[float,:,:]])

    def test_all3_(self):
        self.run_test("def np_all3_(a): return a.all(1)", numpy.arange(10).reshape(2,5), np_all3_=[NDArray[int,:,:]])

    def test_all4_(self):
        self.run_test("def np_all4_(a): return a.all(0)", numpy.ones(10).reshape(2,5), np_all4_=[NDArray[float,:,:]])

    def test_all5_(self):
        self.run_test("def np_all5_(a): return a.all(0)", numpy.arange(10), np_all5_=[NDArray[int,:]])

    def test_all6_(self):
        self.run_test("def np_all6_(a): return a.all().all()", numpy.arange(10), np_all6_=[NDArray[int,:]])

    def test_all7_(self):
        self.run_test("def np_all7_(a): return a.all().all(0)", numpy.arange(10), np_all7_=[NDArray[int,:]])

    def test_transpose_(self):
        self.run_test("def np_transpose_(a): return a.transpose()", numpy.arange(24).reshape(2,3,4), np_transpose_=[NDArray[int,:,:,:]])

    def test_transpose_expr(self):
        self.run_test("def np_transpose_expr(a): return (a + a).transpose()", numpy.ones(24).reshape(2,3,4), np_transpose_expr=[NDArray[float,:,:,:]])

    def test_transpose_expr2(self):
        self.run_test("def np_transpose_expr2(a): import numpy as np; return np.conj(a).T", 1j * numpy.ones(6).reshape(2,3), np_transpose_expr2=[NDArray[complex,:,:]])

    def test_transpose2_(self):
        self.run_test("def np_transpose2_(a): return a.transpose((2,0,1))", numpy.arange(24).reshape(2,3,4), np_transpose2_=[NDArray[int,:,:,:]])

    def test_transpose3_(self):
        self.run_test("def np_transpose3_(a): return a.transpose(2,1,0)", numpy.arange(24).reshape(2,3,4), np_transpose3_=[NDArray[int,:,:,:]])

    def test_alen0(self):
        self.run_test("def np_alen0(a): from numpy import alen ; return alen(a)", numpy.ones((5,6)), np_alen0=[NDArray[float,:,:]])

    def test_alen1(self):
        self.run_test("def np_alen1(a): from numpy import alen ; return alen(-a)", numpy.ones((5,6)), np_alen1=[NDArray[float,:,:]])

    def test_allclose0(self):
        self.run_test("def np_allclose0(a): from numpy import allclose ; return allclose([1e10,1e-7], a)", [1.00001e10,1e-8], np_allclose0=[List[float]])

    def test_allclose1(self):
        self.run_test("def np_allclose1(a): from numpy import allclose; return allclose([1e10,1e-8], +a)", numpy.array([1.00001e10,1e-9]), np_allclose1=[NDArray[float,:]])

    def test_allclose2(self):
        self.run_test("def np_allclose2(a): from numpy import array, allclose; return allclose(array([1e10,1e-8]), a)", numpy.array([1.00001e10,1e-9]), np_allclose2=[NDArray[float,:]])

    def test_allclose3(self):
        self.run_test("def np_allclose3(a): from numpy import allclose; return allclose(a, a)", [1.0, numpy.nan], np_allclose3=[List[float]])

    def test_allclose4(self):
        """ Check allclose behavior with infinity values. """
        self.run_test("""
            def np_allclose4(a):
                from numpy import array, allclose
                return allclose(array([-float('inf'), float('inf'),
                                       -float('inf')]), a)""",
                      numpy.array([float("inf"), float("inf"), -float('inf')]),
                      np_allclose4=[NDArray[float,:]])

    def test_alltrue0(self):
        self.run_test("def np_alltrue0(b): from numpy import alltrue ; return alltrue(b)", numpy.array([True, False, True, True]), np_alltrue0=[NDArray[bool,:]])

    def test_alltrue1(self):
        self.run_test("def np_alltrue1(a): from numpy import alltrue ; return alltrue(a >= 5)", numpy.array([1, 5, 2, 7]), np_alltrue1=[NDArray[int,:]])

    def test_count_nonzero0(self):
        self.run_test("def np_count_nonzero0(a): from numpy import count_nonzero; return count_nonzero(a)",
                      numpy.array([[-1, -5, -2, 7], [9, 3, 0, -0]]), np_count_nonzero0=[NDArray[int,:,:]])

    def test_count_nonzero1(self):
        self.run_test("def np_count_nonzero1(a): from numpy import count_nonzero; return count_nonzero(a)",
                       numpy.array([-1, 5, -2, 0]), np_count_nonzero1=[NDArray[int,:]])

    def test_count_nonzero2(self):
        self.run_test("def np_count_nonzero2(a): from numpy import count_nonzero; return count_nonzero(a)",
                      numpy.array([-1., 0., -2., -1e-20]), np_count_nonzero2=[NDArray[float,:]])

    def test_count_nonzero3(self):
        self.run_test("def np_count_nonzero3(a): from numpy import count_nonzero; return count_nonzero(a)",
                      numpy.array([[0, 2, 0., 4 + 1j], [0.+0.j, 0.+4j, 1.+0j, 1j]]), np_count_nonzero3=[NDArray[complex,:,:]])

    def test_count_nonzero4(self):
        self.run_test("def np_count_nonzero4(a): from numpy import count_nonzero; return count_nonzero(a)",
                      numpy.array([[True, False], [False, False]]), np_count_nonzero4=[NDArray[bool,:,:]])

    def test_count_nonzero5(self):
        self.run_test("def np_count_nonzero5(a): from numpy import count_nonzero; return count_nonzero(a*2)",
                      numpy.array([[-1, -5, -2, 7], [9, 3, 0, -0]]), np_count_nonzero5=[NDArray[int,:,:]])


    def test_isclose0(self):
        self.run_test("def np_isclose0(u): from numpy import isclose; return isclose(u, u)",
                      numpy.array([[-1.01, 1e-10+1e-11, -0, 7., float('NaN')], [-1.0, 1e-10, 0., 7., float('NaN')]]),
                      np_isclose0=[NDArray[float,:,:]])

    def test_isclose1(self):
        self.run_test("def np_isclose1(u, v): from numpy import isclose; return isclose(u, v, 1e-19, 1e-16)",
                      numpy.array([-1.01, 1e-10+1e-11, float("inf"), 7.]),
                      numpy.array([9., 1e-10, float("inf"), float('NaN')]),
                      np_isclose1=[NDArray[float,:], NDArray[float,:]])

    def test_isclose2(self):
        self.run_test("def np_isclose2(u,v): from numpy import isclose; return isclose(u, v, 1e-16, 1e-19)",
                      numpy.array([-1.01, 1e-10+1e-11, -0, 7., float('NaN')]),
                      numpy.array([-1., 1e-10+2e-11, -0, 7.1, float('NaN')]),
                      np_isclose2=[NDArray[float,:], NDArray[float,:]])

    def test_isclose3(self):
        self.run_test("def np_isclose3(u): from numpy import isclose; return isclose(u, u)",
                      numpy.array([9.+3j, 1e-10, 1.1j, float('NaN')]),
                      np_isclose3=[NDArray[complex,:]])

    def test_isclose4(self):
        self.run_test("def np_isclose4(u,v): from numpy import isclose; return isclose(u, v)",
                      numpy.array([True, False, True, True, False]),
                      numpy.array([True, False, False, True, True]),
                      np_isclose4=[NDArray[bool,:], NDArray[bool,:]])

    def test_isclose5(self):
        self.run_test("def np_isclose5(u,v): from numpy import isclose; return isclose(u, v)",
                      1e-10,
                      1e-10+1e-11,
                      np_isclose5=[float, float])

    def test_isclose6(self):
        self.run_test("def np_isclose6(u, v): from numpy import isclose; return isclose(u, v, 1e-19, 1e-16)",
                      numpy.array([[-float("inf"), 1e-10+1e-11, -0, 7.],[9., 1e-10, 0., float('NaN')]]),
                      numpy.array([float("inf"), 1e-10, 0., float('NaN')]),
                      np_isclose6=[NDArray[float,:,:], NDArray[float,:]])

    def test_isclose7(self):
        self.run_test("def np_isclose7(u, v): from numpy import isclose; return isclose(u, v, 1e-19, 1e-16)",
                      numpy.array([9., 1e-10, 0., float('NaN')]),
                      numpy.array([[-1.01, 1e-10+1e-11, -0, 7.],[9., 1e-10, 0., float('NaN')]]),
                      np_isclose7=[NDArray[float,:], NDArray[float,:,:]])

    def test_remainder0(self):
        self.run_test("def np_remainder0(u, v): from numpy import remainder; return remainder(u, v)",
                      numpy.array([9., 9., -9., -9.]),
                      numpy.array([2., -2., 2., -2.]),
                      np_remainder0=[NDArray[float,:], NDArray[float,:]])

    def test_numpy_ones_list(self):
        self.run_test(
            "def np_ones_list(u): from numpy import ones; return ones([u,u])",
            2,
            np_ones_list=[int])



def omp_for_lastprivate():
    sum = 0
    i0 = -1
    LOOPCOUNT = 1000
    if 'omp parallel':
        sum0 = 0
        'omp for schedule(static,7) lastprivate(i0)'
        for i in range(1, LOOPCOUNT + 1):
            sum0 += i
            i0 = i
        'omp critical'
        sum+=sum0
    known_sum = (LOOPCOUNT * (LOOPCOUNT + 1)) / 2
    return (sum == known_sum) and (i0 == LOOPCOUNT)



import unittest
from pythran.tests import TestFromDir
import os


class TestRosetta(TestFromDir):

    path = os.path.join(os.path.dirname(__file__),"rosetta")

TestRosetta.populate(TestRosetta)

if __name__ == '__main__':
    unittest.main()



from pythran.tests import TestEnv
from pythran.typing import List

class TestBlas(TestEnv):

    def test_naive_matrix_multiply(self):
        code="""
def matrix_multiply(m0, m1):
    new_matrix = []
    for i in range(len(m0)):
        new_matrix.append([0]*len(m1[0]))

    for i in range(len(m0)):
        for j in range(len(m1[0])):
            for k in range(len(m1)):
                new_matrix[i][j] += m0[i][k]*m1[k][j]
    return new_matrix"""
        self.run_test(code, [[0,1],[1,0]], [[1,2],[2,1]], matrix_multiply=[List[List[int]],List[List[int]]])




#pythran export CROTG(complex, complex, float, complex)
#runas CROTG(complex(1,2),complex(5,6),3.4,complex(10,-3))
import math

def CROTG(CA,CB,C=0,S=0):
#     .. Scalar Arguments ..
#      COMPLEX CA,CB,S
#      REAL C
#     ..
#
#  Purpose
#  =======
#
#  CROTG determines a complex Givens rotation.
#
#  =====================================================================
#
#     .. Local Scalars ..
#      COMPLEX ALPHA
#      REAL NORM,SCALE
#     ..
    if (abs(CA) == 0.):
        C = 0.
        S = complex(1.,0.)
        CA = CB
    else:
        SCALE = abs(CA) + abs(CB)
        NORM = SCALE*math.sqrt((abs(CA/SCALE))**2+ (abs(CB/SCALE))**2)
        ALPHA = CA/abs(CA)
        C = abs(CA)/NORM
        S = ALPHA*(CB.conjugate())/NORM
        CA = ALPHA*NORM
    return (CA,CB,C,S)






#pythran export list_comp(int list list)
#runas list_comp([[], [], [1]])

def foo(cc, x, y):
    for a in cc:
        if a:
            return True
        return False

def list_comp(cc):
    return [(x,y) for x in range(1) for y in range(2) if foo(cc, x, y)]



#from http://rosettacode.org/wiki/Pythagorean_triples#Python
#pythran export triples(int)
#runas triples(10)
#runas triples(100)
#runas triples(1000)
#runas triples(10000)
#runas triples(100000)

from functools import reduce

def triples(lim, a = 3, b = 4, c = 5):
    l = a + b + c
    if l > lim: return (0, 0)
    return reduce(lambda x, y: (x[0] + y[0], x[1] + y[1]), [
            (1, int(lim / l)),
            triples(lim,  a - 2*b + 2*c,  2*a - b + 2*c,  2*a - 2*b + 3*c),
            triples(lim,  a + 2*b + 2*c,  2*a + b + 2*c,  2*a + 2*b + 3*c),
            triples(lim, -a + 2*b + 2*c, -2*a + b + 2*c, -2*a + 2*b + 3*c) ])



#from: http://stackoverflow.com/questions/2196693/improving-numpy-performance
#pythran export specialconvolve(uint32 [][])
#runas import numpy as np ; r = np.arange(100*100, dtype=np.uint32).reshape(100,100); specialconvolve(r)
#bench import numpy as np ; r = np.arange(1000*1000, dtype=np.uint32).reshape(1000,1000); specialconvolve(r)

def specialconvolve(a):
    # sorry, you must pad the input yourself
    rowconvol = a[1:-1,:] + a[:-2,:] + a[2:,:]
    colconvol = rowconvol[:,1:-1] + rowconvol[:,:-2] + rowconvol[:,2:] - 9*a[1:-1,1:-1]
    return colconvol



#runas mandel(20,0.,0., 8)
#bench mandel(400,0.,0., 75)
#pythran export mandel(int, float, float, int)
def mandel(size, x_center, y_center, max_iteration):
    out= [ [ 0 for i in range(size) ] for j in range(size) ]
    for i in range(size):
        "omp parallel for"
        for j in range(size):
            x,y = ( x_center + 4.0*float(i-size/2)/size,
                      y_center + 4.0*float(j-size/2)/size
                    )

            a,b = (0.0, 0.0)
            iteration = 0

            while (a**2 + b**2 <= 4.0 and iteration < max_iteration):
                a,b = a**2 - b**2 + x, 2*a*b + y
                iteration += 1
            if iteration == max_iteration:
                color_value = 255
            else:
                color_value = iteration*10 % 255
            out[i][j]=color_value
    return out



#runas: import numpy as np ; N = 500 ; X = np.random.randn(N,N,3); laplacien(X)
#pythran export laplacien(float64[][][3])

import numpy as np
def laplacien(image):
    out_image = np.abs(4*image[1:-1,1:-1] -
                                   image[0:-2,1:-1] - image[2:,1:-1] -
                                   image[1:-1,0:-2] - image[1:-1,2:])
    valmax = np.max(out_image)
    valmax = max(1.,valmax)+1.E-9
    out_image /= valmax
    return out_image



#pythran export QuickSort(int list)
#runas QuickSort(list(range(10)))
#bench a = list(range(200000)); QuickSort(a)

# swap two value of the list
def swap (l, idx1, idx2):
    if (idx1 != idx2):
        tmp = l[idx1]
        l[idx1] = l[idx2]
        l[idx2] = tmp

# partition the list using the value at pivot index size / 2 and return the
# new pivot index
def partition (l):
    size = len (l)
    # the pivot indfex
    pivot_idx = size // 2
    # the pivot value
    val = l[pivot_idx]
    # the idx of last unsorted elemet
    idx = size - 1
    # move the pivot to the end
    if (pivot_idx != idx):
        swap (l, pivot_idx, idx)

    # the pivot must stay at the end until the final swap
    idx = idx - 1
    # go through the list of the elements to be sorted
    i = 0
    while (i <= idx):
        if (l[i] > val) :
            while ((l[idx] > val) and (idx > i)):
                idx = idx - 1
            if (idx != i):
                swap (l, i, idx)
                idx = idx - 1
            else:
                break
        i = i+1
    # finally bring the pivot at its final place
    assert ((idx == i) or (idx + 1 == i))
    swap (l, i, size - 1)
    return i

def QuickSort (l):
    size = len (l)
    if size > 1:
        # Get the lists of bigger and smaller items and final position of pivot
        idx = partition (l)
        l1 = []
        l2 = []
        for i in range (0, idx):
            l1.append (l[i])
        for i in range (idx, size):
            l2.append (l[i])
        # Recursively sort elements smaller than the pivot
        QuickSort(l1);
        # Recursively sort elements at least as big as the pivot
        QuickSort(l2);
        for i in range (0, len (l1)):
            l[i] = l1[i]
        for i in range (0, len (l2)):
            l[len (l1) + i] = l2[i]
    return l



#pythran export approximated_callgraph(int)
#runas approximated_callgraph(100)
#bench approximated_callgraph(250)
def call(i, j):
    return i+j

def approximated_callgraph(size):
    out= list()
    for i in range(size):
        out.append(list(map(lambda j:call(i, j), range(size))))
    return out



import numpy
import pytest
import sys
import unittest

from pythran.tests import TestEnv
from pythran.typing import *

class TestBase(TestEnv):
    def test_pass(self):
        self.run_test("def pass_(a):pass", 1, pass_=[int])

    def test_empty_return(self):
        self.run_test("def empty_return(a,b,c):return", 1,1.,True, empty_return=[int,float,bool])

    def test_identity(self):
        self.run_test("def identity(a): return a", 1.5, identity=[float])

    def test_compare(self):
        self.run_test("def compare(a,b,c):\n if a < b < c: return a\n else: return b != c", 1,2,3, compare=[int, int, int])

    def test_arithmetic(self):
        self.run_test("def arithmetic(a,b,c): return a+b*c", 1,2,3.3, arithmetic=[int,int, float])

    def test_boolop(self):
        self.run_test("def boolop(a,b,c): return a and b or c", True, True, False, boolop=[bool,bool, bool])

    def test_operator(self):
        self.run_test("def operator_(a,b,c): return (a+b-b*a//(a%b)**(a<<a>>b|b^a&a//b))/c",1,2,3., operator_=[int,int, float])

    def test_unaryop(self):
        self.run_test("def unaryop(a): return not(~(+(-a)))", 1, unaryop=[int])

    def test_expression(self):
        self.run_test("def expression(a,b,c): a+b*c", 1,2,3.3, expression=[int,int, float])

    def test_recursion1(self):
        code="""
def fibo(n): return n if n <2 else fibo(n-1) + fibo(n-2)
def fibo2(n): return fibo2(n-1) + fibo2(n-2) if n > 1 else n
"""
        self.run_test(code, 4, fibo=[int])

    def test_recursion2(self):
        code="""
def fibo(n): return n if n <2 else fibo(n-1) + fibo(n-2)
def fibo2(n): return fibo2(n-1) + fibo2(n-2) if n > 1 else n
"""
        self.run_test(code, 4., fibo2=[float])

    def test_manual_list_comprehension(self):
        self.run_test("def f(l):\n ll=list()\n for k in l:\n  ll+=[k]\n return ll\ndef manual_list_comprehension(l): return f(l)", [1,2,3], manual_list_comprehension=[List[int]])

    def test_list_comprehension(self):
        self.run_test("def list_comprehension(l): return [ x*x for x in l ]", [1,2,3], list_comprehension=[List[int]])

    def test_dict_comprehension(self):
        self.run_test("def dict_comprehension(l): return { i: 1 for i in l if len(i)>1 }", ["1","12","123"], dict_comprehension=[List[str]])

    def test_filtered_list_comprehension(self):
        self.run_test("def filtered_list_comprehension(l): return [ x*x for x in l if x > 1 if x <10]", [1,2,3], filtered_list_comprehension=[List[int]])

    def test_multilist_comprehension(self):
        self.run_test("def multilist_comprehension(l): return [ x*y for x in l for y in l]", [1,2,3], multilist_comprehension=[List[int]])

    def test_zipped_list_comprehension(self):
        self.run_test("def zipped_list_comprehension(l): return [ x*y for x,y in zip(l,l) ]", [1,2,3], zipped_list_comprehension=[List[int]])

    def test_zip(self):
        self.run_test("def zip_(l0,l1): return list(zip(l0,l1))", [1,2,3],["one", "two", "three"], zip_=[List[int], List[str]])

    def test_multizip(self):
        self.run_test("def multizip(l0,l1): return list(zip(l0,zip(l0,l1)))", [1,2,3],["one", "two", "three"], multizip=[List[int], List[str]])

    def test_reduce(self):
        self.run_test("def reduce_(l): from functools import reduce; return reduce(lambda x,y:x+y, l)", [0.,1.1,2.2,3.3], reduce_=[List[float]])

    def test_another_reduce(self):
        code = '''
        def another_reduce(l0,l1):
            from functools import reduce
            return reduce(lambda x,y:x+y[0]+y[1], zip(l0, l1),0)
        '''
        self.run_test(code, [0.4,1.4,2.4,3.4], [0.,1.1,2.2,3.3], another_reduce=[List[float], List[float]])

    def test_sum(self):
        self.run_test("def sum_(l): return sum(l)", [0.,1.1,2.2,3.3], sum_=[List[float]])

    def test_multisum(self):
        self.run_test("def multisum(l0, l1): return sum(l0) + sum(l1)", [0.,1.1,2.2,3.3],[1,2,3], multisum=[List[float], List[int]])

    def test_max(self):
        self.run_test("def max_(l):return max(l)", [ 1.1, 2.2 ], max_=[List[float]])

    def test_multimax(self):
        self.run_test("def multimax(l,v):return max(v,max(l))", [ 1.1, 2.2 ], 1, multimax=[List[float],int])

    def test_min(self):
        self.run_test("def min_(l):return min(l)", [ 1.1, 2.2 ], min_=[List[float]])

    def test_multimin(self):
        self.run_test("def multimin(l,v):return min(v,min(l))", [ 1.1, 2.2 ], 3, multimin=[List[float],int])

    def test_map(self):
        self.run_test("def map_(l0, l1, v): return list(map(lambda x,y:x*v+y, l0, l1))", [0,1,2], [0.,1.1,2.2], 2, map_=[List[int], List[float], int])

    def test_multimap(self):
        self.run_test("def multimap(l0, l1, v): return list(map(lambda x,y:x*v+y, l0, map(lambda z:z+1,l1)))", [0,1,2], [0.,1.1,2.2], 2, multimap=[List[int], List[float], int])

    def test_intrinsic_map(self):
        self.run_test("def intrinsic_map(l): return list(map(max,l))",[[0,1,2],[2,0,1]], intrinsic_map=[List[List[int]]])

    def test_range1(self):
        self.run_test("def range1_(e): return list(range(e))", 3, range1_=[int])

    def test_range2(self):
        self.run_test("def range2_(b,e): return list(range(b,e))", 1, 3, range2_=[int,int])

    def test_range3(self):
        self.run_test("def range3_(b,e,s): return list(range(b,e,s))", 8,3,-2, range3_=[int,int,int])

    def test_range4(self):
        self.run_test("def range4_(b,e,s): return list(range(b,e,s))", 8,2,-2, range4_=[int,int,int])

    def test_range5(self):
        self.run_test("def range5_(b,e,s): return list(range(b,e,s))", 3,8,1, range5_=[int,int,int])

    def test_range6(self):
        self.run_test("def range6_(b,e,s): return list(range(b,e,s))", 3,8,3, range6_=[int,int,int])

    def test_range7(self):
        self.run_test("def range7_(b,e,s): return list(range(b,e,s))", 3,9,3, range7_=[int,int,int])

    def test_rrange1(self):
        self.run_test("def rrange1_(e): return list(reversed(range(e)))", 3, rrange1_=[int])

    def test_rrange2(self):
        self.run_test("def rrange2_(b,e): return set(reversed(range(b,e)))", 1, 3, rrange2_=[int,int])

    def test_rrange3(self):
        self.run_test("def rrange3_(b,e,s): return list(reversed(range(b,e,s)))", 8,3,-2, rrange3_=[int,int,int])

    def test_rrange4(self):
        self.run_test("def rrange4_(b,e,s): return set(reversed(range(b,e,s)))", 8,2,-2, rrange4_=[int,int,int])

    def test_rrange5(self):
        self.run_test("def rrange5_(b,e,s): return list(reversed(range(b,e,s)))", 3,8,1, rrange5_=[int,int,int])

    def test_rrange6(self):
        self.run_test("def rrange6_(b,e,s): return set(reversed(range(b,e,s)))", 3,8,3, rrange6_=[int,int,int])

    def test_rrange7(self):
        self.run_test("def rrange7_(b,e,s): return list(reversed(range(b,e,s)))", 3,9,3, rrange7_=[int,int,int])

    def test_multirange(self):
        self.run_test("def multirange(i): return list(map(lambda x,y:y*x//2, range(1,i), range(i,1,-1)))", 3, multirange=[int])

    def test_xrange1(self):
        self.run_test("def xrange1_(e): return list(range(e))", 3, xrange1_=[int])

    def test_xrange2(self):
        self.run_test("def xrange2_(b,e): return list(range(b,e))", 1, 3, xrange2_=[int,int])

    def test_xrange3(self):
        self.run_test("def xrange3_(b,e,s): return list(range(b,e,s))", 8,3,-2, xrange3_=[int,int,int])

    def test_xrange4(self):
        self.run_test("def xrange4_(b,e,s): return list(range(b,e,s))", 3,8,1, xrange4_=[int,int,int])

    def test_xrange5(self):
        self.run_test("def xrange5_(e): return max(range(e))", 3, xrange5_=[int])

    def test_multixrange(self):
        self.run_test("def multixrange(i): return list(map(lambda x,y:y*x//2, range(1,i), range(i,1,-1)))", 3, multixrange=[int])

    def test_print(self):
        self.run_test("def print_(a,b,c,d): print(a,b,c,d,'e',1.5)", [1.,2.,3.1],3,True, "d", print_=[List[float], int, bool, str])

    def test_print_tuple(self):
        self.run_test("def print_tuple(a,b,c,d): t = (a,b,c,d,'e',1.5,); print(t)", [1.,2.,3.1],3,True, "d", print_tuple=[List[float], int, bool, str])

    def test_fstring_noarg(self):
        self.run_test("def fstring_noarg(): return f'hello'", fstring_noarg=[])

    def test_fstring_1arg(self):
        self.run_test("def fstring_1arg(a): return f'{a:s}'", "a", fstring_1arg=[str])

    def test_fstring(self):
        self.run_test("def fstring(a,b,c): return f'a: {a: 4d}; b: {b:.2f}; c: {c:s}'", 2, 6.28, "c", fstring=[int, float, str])

    def test_assign(self):
        self.run_test("def assign(a): b=2*a ; return b", 1, assign=[int])

    def test_multiassign(self):
        self.run_test("def multiassign(a):\n c=b=a\n return c", [1], multiassign=[List[int]])

    def test_divmax(self):
        self.run_test("def divmax_(a): b=4.*a;c=b/2;return max(c,b)", 1, divmax_=[int])

    @unittest.skip("impossible to handle max(int, float) without conversion")
    def test_divmax_float(self):
        self.run_test("def divmax_float_(a): b=2*a;c=b/2;return max(c,b)", 1, divmax_float_=[int])

    def test_if(self):
        self.run_test("def if_(a,b):\n if a>b: return a\n else: return b", 1, 1.1, if_=[int, float])

    def test_while(self):
        self.run_test("def while_(a):\n while(a>0): a-=1\n return a", 8, while_=[int])

    def test_for(self):
        self.run_test("def for_(l):\n s=0\n for i in l:\n  s+=i\n return s", [0,1,2], for_=[List[int]])

    def test_declarations(self):
        code = """
def declarations():
    if True:
        a=0
        while a <3:
            b = 1
            a = b + a
    else:
        a=1
    return a + b
"""
        self.run_test(code, declarations=[])

    def test_lambda(self):
        code = """
def lambda_():
    l=lambda x,y: x+y
    return l(1,2) + l(1.2,2)
"""
        self.run_test(code, lambda_=[])

    def test_multidef1(self):
        self.run_test("def def10(): pass\ndef def11(): def10()", def11=[])

    def test_multidef2(self):
        self.run_test("def def21(): def20()\ndef def20(): pass", def21=[])

    def test_multidef3(self):
        self.run_test("def def31(): return 1\ndef def30(): return def31()", def31=[])

    def test_multidef4(self):
       self.run_test("def def41(): return def40()\ndef def40(): return 1", def41=[])

    def test_tuple(self):
        self.run_test("def tuple_(t): return t[0]+t[1]", (0,1), tuple_=[Tuple[int, int]])

    def test_nested_list_comprehension(self):
        self.run_test("def nested_list_comprehension(): return [ [ x+y for x in range(10) ] for y in range(20) ]", nested_list_comprehension=[])

    def test_delete(self):
        self.run_test("def delete_(v): del v", 1, delete_=[int])

    def test_continue(self):
        self.run_test("def continue_():\n for i in range(3):continue\n return i", continue_=[])

    def test_break(self):
        self.run_test("def break_():\n for i in range(3):break\n return i", break_=[])

    def test_assert(self):
        with self.assertRaises(AssertionError):
            self.run_test("def assert_(i): assert i > 0", -1, assert_=[int])

    def test_assert_with_msg(self):
        self.run_test("def assert_with_msg(i): assert i > 0, 'hell yeah'", 1, assert_with_msg=[int])

    def test_import_from(self):
        self.run_test("def import_from(): from math import cos ; return cos(1.)", import_from=[])

    def test_len(self):
        self.run_test("def len_(i,j,k): return len(i)+len(j)+len(k)", "youpi", [1,2],[], len_=[str,List[int], List[float]])

    def test_in_string(self):
        self.run_test("def in_string(i,j): return i in j", "yo", "youpi", in_string=[str,str])

    def test_not_in_string(self):
        self.run_test("def not_in_string(i,j): return i not in j", "yo", "youpi", not_in_string=[str,str])

    def test_in_list(self):
        self.run_test("def in_list(i,j): return i in j", 1, [1,2,3], in_list=[int,List[int]])

    def test_not_in_list(self):
        self.run_test("def not_in_list(i,j): return i not in j", False, [True, True, True], not_in_list=[bool,List[bool]])

    def test_subscript(self):
        self.run_test("def subscript(l,i): l[0]=l[0]+l[i]", [1], 0, subscript=[List[int], int])

    def test_nested_lists(self):
        self.run_test("def nested_lists(l,i): return l[0][i]", [[1]], 0, nested_lists=[List[List[int]],int])

    def test_nested_tuples(self):
        self.run_test("def nested_tuples(l,i): return l[i][1]", [(0.1,1,)], 0, nested_tuples=[List[Tuple[float,int]],int])

    def test_return_empty_list(self):
        self.run_test("def return_empty_list(): return list()", return_empty_list=[])

    def test_empty_list(self):
        self.run_test("def empty_list(): a=[]", empty_list=[])

    def test_multi_list(self):
        self.run_test("def multi_list(): return [[[2.0],[1,2,3]],[[2.0],[1,2,3]]]", multi_list=[])

    def test_empty_tuple(self):
        self.run_test("def empty_tuple(): a=()", empty_tuple=[])

    def test_multi_tuple(self):
        self.run_test("def multi_tuple(): return (1,('e',2.0),[1,2,3])", multi_tuple=[])

    def test_augmented_assign0(self):
        self.run_test("def augmented_assign0(a):\n a+=1.5\n return a", 12, augmented_assign0=[int])

    def test_augmented_assign1(self):
        self.run_test("def augmented_assign1(a):\n a-=1.5\n return a", 12, augmented_assign1=[int])

    def test_augmented_assign2(self):
        self.run_test("def augmented_assign2(a):\n a*=1.5\n return a", 12, augmented_assign2=[int])

    def test_augmented_assign3(self):
        self.run_test("def augmented_assign3(a):\n a/=1.5\n return a", 12, augmented_assign3=[int])

    def test_augmented_assign4(self):
        self.run_test("def augmented_assign4(a):\n a %= 5\n return a", 12, augmented_assign4=[int])

    def test_augmented_assign5(self):
        self.run_test("def augmented_assign5(a):\n a//=2\n return a", 12, augmented_assign5=[int])

    def test_augmented_assign6(self):
        self.run_test("def augmented_assign6(a):\n a**=5\n return a", 12, augmented_assign6=[int])

    def test_augmented_assign7(self):
        self.run_test("def augmented_assign7(a):\n a<<=1\n return a", 12, augmented_assign7=[int])

    def test_augmented_assign8(self):
        self.run_test("def augmented_assign8(a):\n a>>=1\n return a", 12, augmented_assign8=[int])

    def test_augmented_assign9(self):
        self.run_test("def augmented_assign9(a):\n a^=1\n return a", 12, augmented_assign9=[int])

    def test_augmented_assignA(self):
        self.run_test("def augmented_assignA(a):\n a|=1\n return a", 12, augmented_assignA=[int])

    def test_augmented_assignB(self):
        self.run_test("def augmented_assignB(a):\n a&=1\n return a", 12, augmented_assignB=[int])

    def test_augmented_list_assign(self):
        self.run_test("def augmented_list_assign(l):\n a=list()\n a+=l\n return a", [1,2], augmented_list_assign=[List[int]])

    def test_initialization_list(self):
        self.run_test("def initialization_list(): return [1, 2.3]", initialization_list=[])

    def test_multiple_assign(self):
        self.run_test("def multiple_assign():\n a=0 ; b = a\n a=1.5\n return a, b", multiple_assign=[])

    def test_multiple_return1(self):
        self.run_test("def multiple_return1(a):\n if True:return 1\n else:\n  return a", 2,  multiple_return1=[int])

    def test_multiple_return2(self):
        self.run_test("def multiple_return2(a):\n if True:return 1\n else:\n  b=a\n  return b", 2,  multiple_return2=[int])

    def test_multiple_return3(self):
        self.run_test("def multiple_return3(a):\n if True:return 1\n else:\n  b=a\n  return a+b", 2,  multiple_return3=[int])

    def test_id(self):
        self.run_test("def id_(a):\n c=a\n return id(a)==id(c)", [1,2,3], id_=[List[int]])

    def test_delayed_max(self):
        self.run_test("def delayed_max(a,b,c):\n m=max\n return m(a,b) + m(b,c)", 1, 2, 3.5, delayed_max=[int, int, float])

    def test_slicing(self):
        self.run_test("def slicing(l): return l[0:1] + l[:-1]",[1,2,3,4], slicing=[List[int]])

    def test_not_so_deep_recursive_calls(self):
        code="""
def a(i): return b(i)
def b(i): return b(a(i-1)) if i else i
def not_so_deep_recursive_calls(i):return b(i)"""
        self.run_test(code,3, not_so_deep_recursive_calls=[int])

    def test_deep_recursive_calls(self):
        code="""
def a(i): return a(i-1) + b(i) if i else i
def b(i): return b(i-1)+a(i-1) if i else c(i-1) if i+1 else i
def c(i): return c(i-1) if i>0 else 1
def deep_recursive_calls(i):a(i)+b(i) +c(i)"""
        self.run_test(code,3, deep_recursive_calls=[int])

    def test_dummy_nested_def(self):
        code="""
def dummy_nested_def(a):
    def the_dummy_nested_def(b):return b
    return the_dummy_nested_def(a)"""
        self.run_test(code,3, dummy_nested_def=[int])

    def test_nested_def(self):
        code="""
def nested_def(a):
    def the_nested_def(b):return a+b
    return the_nested_def(3)"""
        self.run_test(code,3, nested_def=[int])

    def test_none(self):
        self.run_test("def none_(l):\n if len(l)==0: return\n else: return l", [], none_=[List[int]])

    def test_import(self):
        self.run_test("import math\ndef import_(): return math.cos(1)", import_=[])

    def test_local_import(self):
        self.run_test("def local_import_(): import math;return math.cos(1)", local_import_=[])

    def test_abs(self):
        """ Check builtins.abs behavior with float. """
        self.run_test("""
        def abs_(a):
            return abs(a)""",
                      -1.3, abs_=[float])

    def test_npabs(self):
        """ Check builtins.abs behavior with numpy.array. """
        self.run_test("""
        def npabs_(a):
            return abs(a)""",
                      numpy.array([-1.3, 2.3, -4]),
                      npabs_=[NDArray[float, :]])

    def test_all(self):
        self.run_test("def all_(a): return all(a)", [True, False, True], all_=[List[bool]])

    def test_any(self):
        self.run_test("def any_(a): return any(a)", [0, 1, 2], any_=[List[int]])

    def test_bin(self):
        self.run_test("def bin_(a): return bin(a)", 54321, bin_=[int])

    def test_bin2(self):
        self.run_test("def bin2_(a): return bin(a)", -543, bin2_=[int])

    @pytest.mark.skipif(sys.platform == "win32",
                        reason="Windows uses long of 32-bit, this test assumes they are 64-bit")
    def test_bin3(self):
        self.run_test("def bin3_(a): return bin(a)", -sys.maxsize - 1, bin3_=[int])

    def test_bin4(self):
        self.run_test("def bin4_(a): return bin(a)", -1, bin4_=[int])

    def test_chr(self):
        self.run_test("def chr_(a): return chr(a)", 42, chr_=[int])

    def test_complex(self):
        self.run_test("def complex_(a): return complex(a)", 1, complex_=[int])

    def test_divmod(self):
        self.run_test("def divmod_(a,b): return divmod(a,b)", 5, 2, divmod_=[int,int])

    def test_enumerate(self):
        self.run_test("def enumerate_(l): return [ x for x in enumerate(l) ]", ["a","b","c"], enumerate_=[List[str]])

    def test_enumerat2(self):
        self.run_test("def enumerate2_(l): return [ x for x in enumerate(l, 3) ]", ["a","b","c"], enumerate2_=[List[str]])

    def test_filter(self):
        self.run_test("def filter_(l): return list(filter(lambda x:x%2, l))", [1,2,3], filter_=[List[int]])

    def test_hex(self):
        self.run_test("def hex_(a): return hex(a)", 18, hex_=[int])

    def test_oct(self):
        self.run_test("def oct_(a): return oct(a)", 18, oct_=[int])

    def test_pow(self):
        self.run_test("def pow_(a): return pow(a,5)", 18, pow_=[int])

    def test_pow_op0(self):
        self.run_test("def pow_op0(a): return a ** 0, a ** 1, a **2, a ** 3, a ** 4, a ** 5, a ** 6, a** 7",
                      18, pow_op0=[int])

    def test_pow_op1(self):
        self.run_test("def pow_op1(a): return a ** -0, a ** -1, a **-2, a ** -3, a ** -4, a ** -5, a ** -6, a** -7",
                      18, pow_op1=[int])

    def test_pow_op2(self):
        self.run_test("def pow_op2(a): return int(a ** a), a ** -a",
                      7, pow_op2=[int])

    def test_reversed(self):
        self.run_test("def reversed_(l): return [x for x in reversed(l)]", [1,2,3], reversed_=[List[int]])

    def test_round(self):
        self.run_test("def round_(v): return round(v) + round(v,2)", 0.1234, round_=[float])

    def test_sorted0(self):
        self.run_test("def sorted0(l): return [x for x in sorted(l)]", [4, 1,2,3], sorted0=[List[int]])

    def test_sorted1(self):
        self.run_test("def sorted1(l): return [x for x in sorted(l, reverse=True)]", [4, 1,2,3], sorted1=[List[int]])

    def test_sorted2(self):
        self.run_test("def sorted2(l): return [x for x in sorted(l, key=lambda x:-x)]", [4, 1,2,3], sorted2=[List[int]])

    def test_sorted3(self):
        self.run_test("def sorted3(l): return [x for x in sorted(l,reverse=True,key=lambda x:-x)]", [4, 1,2,3], sorted3=[List[int]])

    def test_str(self):
        self.run_test("def str_(l): return str(l)", [1,2,3], str_=[List[int]])

    def test_append(self):
        self.run_test("def append(): l=[] ; l.append(1) ; return l", append=[])

    def test_append_in_call(self):
        self.run_test("def call(l):l.append(1.)\ndef append_in_call(): l=[] ; call(l) ; l.append(1) ; return l", append_in_call=[])

    def test_complex_append_in_call(self):
        code="""
def foo(a,b):
	i = 3*b
	if not i in a:
		a.append(i)
def complex_append_in_call(l1,l2):
	b = []
	for x in l1:
		if not x in l2:
			foo(b,x)"""
        self.run_test(code, [1,2,3],[2],complex_append_in_call=[List[int],List[int]])

    def test_complex_number(self):
        code="""
def complex_number():
    c=complex(0,1)
    return c.real + c.imag"""
        self.run_test(code, complex_number=[])

    def test_raise(self):
        self.run_test("def raise_():\n raise RuntimeError('pof')", raise_=[], check_exception=True)

    def test_complex_number_serialization(self):
        self.run_test("def complex_number_serialization(l): return [x+y for x in l for y in l]", [complex(1,0), complex(1,0)], complex_number_serialization=[List[complex]])

    def test_complex_conj(self):
        self.run_test("def complex_conjugate(c): return c.conjugate()", complex(0,1), complex_conjugate=[complex])

    def test_cast(self):
        self.run_test("def cast(i,f): return float(i)+int(f)", 1,1.5, cast=[int, float])

    def test_subscript_assignment(self):
        code="""
def foo(A):
    A[0]=1.5
def subscript_assignment ():
    a=list(range(1))
    foo(a)
    return a[0]"""
        self.run_test(code,subscript_assignment=[])

    def test_conflicting_keywords(self):
        code="""
def export(template):
    return [ new*new for new in template ]"""
        self.run_test(code, [1], export=[List[int]])

    def test_forelse(self):
        code="""
def forelse():
    l=0
    for i in range(10):
        if i > 3:break
        for j in range(10):
            if j > 5:break
            l+=1
        else:
            l*=2
    else:
        l*=3
    return l"""
        self.run_test(code, forelse=[])

    def test_tuples(self):
        self.run_test("def tuples(n): return ((1,2.,'e') , [ x for x in tuple([1,2,n])] )", 1, tuples=[int])

    def test_reversed_slice(self):
        self.run_test("def reversed_slice(l): return l[::-2]", [0,1,2,3,4], reversed_slice=[List[int]])

    def test_shadow_parameters(self):
        code="""
def shadow_parameters(l):
    if False:l=None
    return l"""
        self.run_test(code, [1], shadow_parameters=[List[int]])

    def test_yielder(self):
        code="""
def iyielder(i):
    for k in range(i+18):
        yield k
    return

def yielder():
    f=iyielder(1)
    b=next(f)
    return [i*i for i in f]"""
        self.run_test(code, yielder=[])

    def test_yield_with_default_param(self):
        code="""
def foo(a=1000):
    for i in range(10):
        yield a

def yield_param():
    it = foo()
    return [i for i in it]"""
        self.run_test(code, yield_param=[])

    def test_set(self):
        code="""
def set_(a,b):
    S=set()
    S.add(a)
    S.add(b)
    return len(S)"""
        self.run_test(code, 1,2,set_=[int, int])
    def test_in_set(self):
        code="""
def in_set(a):
    S=set()
    S.add(a)
    return a in S"""
        self.run_test(code, 1.5, in_set=[float])

    def test_return_set(self):
        self.run_test("def return_set(l): return set(l)", [1,2,3,3], return_set=[List[int]])

    def test_import_set(self):
        self.run_test("def import_set(l): l.add(1) ; return l", {0,2}, import_set=[Set[int]])

    def test_raw_set(self):
        self.run_test("def raw_set(): return { 1, 1., 2 }", raw_set=[])

    def test_iter_set(self):
        self.run_test("def iter_set(s):\n l=0\n for k in s: l+=k\n return l", { 1, 2, 3 } , iter_set=[Set[int]])

    def test_set_comprehension(self):
        self.run_test("def set_comprehension(l): return { i*i for i in l }", [1 , 2, 1, 3], set_comprehension=[List[int]])

    def test_slicer(self):
        code="""
def slicer(l):
    l[2:5]=[1,2]
    return l"""
        self.run_test(code,[1,2,3,4,5,6,7,8,9], slicer=[List[int]])

    def test_generator_expression(self):
        code="""
def generator_expression(l):
    return sum(x for x in l if x == 1)"""
        self.run_test(code,[1,1,1,2], generator_expression=[List[int]])

    def test_default_parameters(self):
        code="""
def dp(b,a=1.2):
    return a

def default_parameters():
    a=1
    c=dp(a)
    d=dp(5,"yeah")
    return str(c)+d"""
        self.run_test(code, default_parameters=[])

    def test_import_as(self):
        code="""
from math import cos as COS
def import_as():
    x=.42
    import math as MATH
    return MATH.sin(x)**2 + COS(x)**2"""
        self.run_test(code, import_as=[])

    def test_tuple_unpacking(self):
        self.run_test("def tuple_unpacking(t): a,b = t ; return a, b", (1,"e"), tuple_unpacking=[Tuple[int, str]])

    def test_list_unpacking(self):
        self.run_test("def list_unpacking(t): [a,b] = t ; return a, b", (1,2), list_unpacking=[Tuple[int, int]])

    def test_recursive_attr(self):
        self.run_test("def recursive_attr(): return {1,2,3}.union({1,2}).union({5})", recursive_attr=[])

    def test_range_negative_step(self):
        self.run_test("""def range_negative_step(n):
        o=[]
        for i in range(n, 0, -1): o.append(i)
        return o""", 10, range_negative_step=[int])

    def test_reversed_range_negative_step(self):
        self.run_test("""def reversed_range_negative_step(n):
        o=[]
        for i in reversed(range(n, 0, -1)): o.append(i)
        return o""", 10, reversed_range_negative_step=[int])

    def test_update_empty_list(self):
        self.run_test('''
def update_empty_list(l):
    p = list()
    return p + l[:1]''', list(range(5)), update_empty_list=[List[int]])

    def test_update_list_with_slice(self):
        self.run_test('''
def update_list_with_slice(l):
    p = list()
    for i in range(10):
        p += l[:1]
    return p,i''', list(range(5)), update_list_with_slice=[List[int]])

    def test_add_slice_to_list(self):
        self.run_test('''
def add_slice_to_list(l):
    p = list()
    for i in range(10):
        p = p + l[:1]
    return p,i''', list(range(5)), add_slice_to_list=[List[int]])

    def test_bool_(self):
        self.run_test("def _bool(d): return bool(d)", 3, _bool=[int])

    def test_complex_add(self):
        self.run_test("def complex_add(): a = 1j ; b = 2 ; return a + b", complex_add=[])

    def test_complex_sub(self):
        self.run_test("def complex_sub(): a = 1j ; b = 2 ; return a - b", complex_sub=[])

    def test_complex_mul(self):
        self.run_test("def complex_mul(): a = 1j ; b = 2 ; return a * b", complex_mul=[])

    def test_complex_div(self):
        self.run_test("def complex_div(): a = 1j ; b = 2 ; return a / b", complex_div=[])

    def test_modulo_int0(self):
        self.run_test("def modulo_int0(n): return n%3, (-n)%3",
                      5,
                      modulo_int0=[int])

    def test_modulo_int1(self):
        self.run_test("def modulo_int1(n): return n%3, (-n)%3",
                      3,
                      modulo_int1=[int])

    def test_modulo_float0(self):
        self.run_test("def modulo_float0(n): return n%3, (-n)%3",
                      5.4,
                      modulo_float0=[float])

    def test_modulo_float1(self):
        self.run_test("def modulo_float1(n): return n%3, (-n)%3",
                      3.5,
                      modulo_float1=[float])

    def test_floordiv_int0(self):
        self.run_test("def floordiv_int0(n): return n%3, (-n)%3",
                      5,
                      floordiv_int0=[int])

    def test_floordiv_int1(self):
        self.run_test("def floordiv_int1(n): return n//2, (-n)//2",
                      3,
                      floordiv_int1=[int])

    def test_floordiv_float0(self):
        self.run_test("def floordiv_float0(n): return n//2, (-n)//2",
                      5.4,
                      floordiv_float0=[float])

    def test_floordiv_float1(self):
        self.run_test("def floordiv_float1(n): return n//2, (-n)//2",
                      3.5,
                      floordiv_float1=[float])

    def test_int_base0(self):
        self.run_test("def int_base(x, y): return [int(x0, y0) for x0, y0 in zip(x, y)]",
                      ["11", "11", "16", "FF"],
                      [2, 4, 8, 16],
                      int_base=[List[str], List[int]])

    def test_int_base1(self):
        self.run_test("def int_base_lit(x, y): return int(x, 8), int('A', y)",
                      "14", 16,
                      int_base_lit=[str, int])

    def test_slice0(self):
        self.run_test("""
        def slice0(a, x):
            return a[x], x, slice(1,1,1)""",
                      numpy.array([-1.3, 2.3, -4]),
                      slice(0,2,1),
                      slice0=[NDArray[float, :], slice])



#runas solve()
#unittest.skip recursive generator
#pythran export solve()
def solve():
    '''
    The number, 1406357289, is a 0 to 9 pandigital number because it is made up of each of the digits 0 to 9 in some order, but it also has a rather interesting sub-string divisibility property.
    
    Let d1 be the 1st digit, d2 be the 2nd digit, and so on. In this way, we note the following:
    
    d2 d3 d4 = 406 is divisible by 2
    d3 d4 d5 = 063 is divisible by 3
    d4 d5 d6 = 635 is divisible by 5
    d5 d6 d7 = 357 is divisible by 7
    d6 d7 d8 = 572 is divisible by 11
    d7 d8 d9 = 728 is divisible by 13
    d8 d9 d10= 289 is divisible by 17
    
    Find the sum of all 0 to 9 pandigital numbers with this property.
    '''

    def _combinators(_handle, items, n):
        if n==0:
            yield []
            return
        for i, item in enumerate(items):
            this_one = [ item ]
            for cc in _combinators(_handle, _handle(items, i), n-1):
                yield this_one + cc

    def combinations(items, n):
        ''' take n distinct items, order matters '''
        def skipIthItem(items, i):
            return items[:i] + items[i+1:]
        return _combinators(skipIthItem, items, n)

    def permutations(items):
        ''' take all items, order matters '''
        return combinations(items, len(items))

    def num(l):
        s = 0
        for n in l: s = s * 10 + n
        return s

    def subdiv(l, n): return num(l) % n == 0

    total = 0
    for perm in permutations((0,1,2,3,4,6,7,8,9)):
        perm.insert(5, 5)               # d6 must be 5
        if (subdiv(perm[7:10], 17) and
            subdiv(perm[6:9],  13) and
            subdiv(perm[5:8],  11) and
            subdiv(perm[4:7],   7) and
            subdiv(perm[3:6],   5) and
            subdiv(perm[2:5],   3) and
            subdiv(perm[1:4],   2)):
                total += num(perm)

    return total




def omp_atomic_bitand():
    sum = 0
    LOOPCOUNT = 1000
    logics = [1]*LOOPCOUNT
    "omp parallel for"
    for i in range(LOOPCOUNT):
        "omp atomic"
        sum &= logics[i]
    return sum == 0



import unittest
from pythran.tests import TestEnv
import numpy

from pythran.typing import NDArray, List, Tuple


@TestEnv.module
class TestNumpyFunc2(TestEnv):

    def test_nonzero0(self):
        self.run_test("def np_nonzero0(x): return x.nonzero()", numpy.arange(6), np_nonzero0=[NDArray[int,:]])

    def test_nonzero1(self):
        self.run_test("def np_nonzero1(x): from numpy import nonzero ; return nonzero(x>8)", numpy.arange(6), np_nonzero1=[NDArray[int,:]])

    def test_nonzero2(self):
        self.run_test("def np_nonzero2(x): from numpy import nonzero ; return nonzero(x>0)", numpy.arange(6).reshape(2,3), np_nonzero2=[NDArray[int,:,:]])

    def test_diagflat3(self):
        self.run_test("def np_diagflat3(a): from numpy import diagflat ; return diagflat(a)", numpy.arange(2), np_diagflat3=[NDArray[int,:]])

    def test_diagflat4(self):
        self.run_test("def np_diagflat4(a): from numpy import diagflat ; return diagflat(a,1)", numpy.arange(3), np_diagflat4=[NDArray[int,:]])

    def test_diagflat5(self):
        self.run_test("def np_diagflat5(a): from numpy import diagflat ; return diagflat(a,-2)", numpy.arange(4), np_diagflat5=[NDArray[int,:]])

    def test_diagonal0(self):
        self.run_test("def np_diagonal0(a): return a.diagonal()", numpy.arange(10).reshape(2,5), np_diagonal0=[NDArray[int,:,:]])

    def test_diagonal1(self):
        self.run_test("def np_diagonal1(a): from numpy import diagonal ; return diagonal(a,1)", numpy.arange(9).reshape(3,3), np_diagonal1=[NDArray[int,:,:]])

    def test_diagonal2(self):
        self.run_test("def np_diagonal2(a): from numpy import diagonal ; return diagonal(a,-2)", numpy.arange(9).reshape(3,3), np_diagonal2=[NDArray[int,:,:]])

    def test_diag0(self):
        self.run_test("def np_diag0(a): from numpy import diag ; return diag(a)", numpy.arange(10).reshape(2,5), np_diag0=[NDArray[int,:,:]])

    def test_diag1(self):
        self.run_test("def np_diag1(a): from numpy import diag ; return diag(a,1)", numpy.arange(9).reshape(3,3), np_diag1=[NDArray[int,:,:]])

    def test_diag2(self):
        self.run_test("def np_diag2(a): from numpy import diag ; return diag(a,-2)", numpy.arange(9).reshape(3,3), np_diag2=[NDArray[int,:,:]])

    def test_diag2b(self):
        self.run_test("def np_diag2b(a): from numpy import diag ; return diag(a,-2)", numpy.arange(12).reshape(4,3), np_diag2b=[NDArray[int,:,:]])

    def test_diag3(self):
        self.run_test("def np_diag3(a): from numpy import diag ; return diag(a)", numpy.arange(2), np_diag3=[NDArray[int,:]])

    def test_diag4(self):
        self.run_test("def np_diag4(a): from numpy import diag ; return diag(a,1)", numpy.arange(3), np_diag4=[NDArray[int,:]])

    def test_diag5(self):
        self.run_test("def np_diag5(a): from numpy import diag; return diag(a,-2)", numpy.arange(4), np_diag5=[NDArray[int,:]])

    def test_delete0(self):
        self.run_test("def np_delete0(a): from numpy import delete ; return delete(a, 1)", numpy.array([[1,2,3,4], [5,6,7,8], [9,10,11,12]]), np_delete0=[NDArray[int,:,:]])

    def test_delete1(self):
        self.run_test("def np_delete1(a): from numpy import delete ; return delete(a, [1,3,5])", numpy.array([[1,2,3,4], [5,6,7,8], [9,10,11,12]]), np_delete1=[NDArray[int,:,:]])

    def test_where0(self):
        self.run_test("""def np_where0(a):
    from numpy import arange, where
    b = arange(5, 17).reshape((3,4))
    c = [[0, 1, 1, 1], [0, 0, 1, 1], [1, 0, 0, 0]]
    return where(c , a, b)""", numpy.arange(12).reshape(3,4), np_where0=[NDArray[int,:,:]])

    def test_where1(self):
        self.run_test("""def np_where1(a):
    from numpy import arange, where
    c = [[0, 1, 1, 1], [0, 0, 1, 1], [1, 0, 0, 0]]
    return where(True , a, c)""", numpy.arange(12).reshape(3,4), np_where1=[NDArray[int,:,:]])

    def test_where2(self):
        self.run_test("""def np_where2(a):
    from numpy import arange, where
    c = [[0, 1, 1, 1], [0, 0, 1, 1], [1, 0, 0, 0]]
    return where(False , a, c)""", numpy.arange(12).reshape(3,4), np_where2=[NDArray[int,:,:]])

    def test_where3(self):
        self.run_test("""def np_where3(a):
    from numpy import arange, where
    c = [[0, 1, 1, 1], [0, 0, 1, 1], [1, 0, 0, 0]]
    return where(True , a, 5)""", numpy.arange(12).reshape(3,4), np_where3=[NDArray[int,:,:]])

    def test_where4(self):
        self.run_test("""def np_where4(a):
    from numpy import arange, where
    c = [[0, 1, 1, 1], [0, 0, 1, 1], [1, 0, 0, 0]]
    return where(False , a, 6)""", numpy.arange(12).reshape(3,4), np_where4=[NDArray[int,:,:]])

    def test_where5(self):
        self.run_test("""def np_where5(a):
    from numpy import arange, where
    b = arange(5, 17).reshape((3,4))
    return where(a>5 , a, b)""", numpy.arange(12).reshape(3,4), np_where5=[NDArray[int,:,:]])

    def test_where6(self):
        self.run_test("""def np_where6(a):
    from numpy import arange, where
    return where(a>5 , 1, 2)""", numpy.arange(12).reshape(3,4), np_where6=[NDArray[int,:,:]])

    def test_where7(self):
        self.run_test("""def np_where7(a):
    from numpy import arange, where
    return where(a>5)""", numpy.arange(12).reshape(3,4), np_where7=[NDArray[int,:,:]])

    def test_cumprod_(self):
        self.run_test("def np_cumprod_(a):\n return a.cumprod()", numpy.arange(10), np_cumprod_=[NDArray[int,:]])

    def test_cumprod2_(self):
        self.run_test("def np_cumprod2_(a):\n from numpy import cumprod\n return a.cumprod()", numpy.arange(10).reshape(2,5), np_cumprod2_=[NDArray[int,:,:]])

    def test_cumprod3_(self):
        self.run_test("def np_cumprod3_(a):\n from numpy import cumprod\n return a.cumprod(1)", numpy.arange(10).reshape(2,5), np_cumprod3_=[NDArray[int,:,:]])

    def test_cumprod4_(self):
        self.run_test("def np_cumprod4_(a):\n from numpy import cumprod\n return a.cumprod(0)", numpy.arange(10).reshape(2,5), np_cumprod4_=[NDArray[int,:,:]])

    def test_cumprod5_(self):
        self.run_test("def np_cumprod5_(a):\n from numpy import cumprod\n return a.cumprod(0)", numpy.arange(10), np_cumprod5_=[NDArray[int,:]])

    def test_correlate_1(self):
        self.run_test("def np_correlate_1(a,b):\n from numpy import correlate\n return correlate(a,b)",
                      numpy.arange(10,dtype=float),
                      numpy.arange(12,dtype=float),
                      np_correlate_1=[NDArray[float,:],NDArray[float,:]])
    def test_correlate_2(self):
        self.run_test("def np_correlate_2(a,b):\n from numpy import correlate\n return correlate(a,b)",
                  numpy.arange(12,dtype=float),
                  numpy.arange(10,dtype=float),
                  np_correlate_2=[NDArray[float,:],NDArray[float,:]])

    def test_correlate_3(self):
        self.run_test("def np_correlate_3(a,b):\n from numpy import correlate\n return correlate(a,b,'valid')",
                  numpy.arange(12,dtype=float),
                  numpy.arange(10,dtype=float),
                  np_correlate_3=[NDArray[float,:],NDArray[float,:]])

    def test_correlate_4(self):
        self.run_test("def np_correlate_4(a,b):\n from numpy import correlate\n return correlate(a,b,'same')",
                  numpy.arange(12,dtype=float),
                  numpy.arange(10,dtype=float),
                  np_correlate_4=[NDArray[float,:],NDArray[float,:]])

    def test_correlate_5(self):
        self.run_test("def np_correlate_5(a,b):\n from numpy import correlate\n return correlate(a,b,'same')",
                  numpy.arange(12,dtype=float),
                  numpy.arange(7,dtype=float),
                  np_correlate_5=[NDArray[float,:],NDArray[float,:]])

    def test_correlate_6(self):
        self.run_test("def np_correlate_6(a,b):\n from numpy import correlate\n return correlate(a,b,'full')",
                  numpy.arange(12) + 1j*numpy.arange(12.),
                  numpy.arange(7.) + 1j* numpy.arange(7.),
                  np_correlate_6=[NDArray[complex,:],NDArray[complex,:]])

    def test_correlate_7(self):
        dtype = numpy.float32
        self.run_test("def np_correlate_7(a,b):\n from numpy import correlate\n return correlate(a,b,'full')",
                  numpy.arange(12).astype(dtype) + 1j*numpy.arange(12).astype(dtype),
                  numpy.arange(7).astype(dtype) + 1j* numpy.arange(7).astype(dtype),
                  np_correlate_7=[NDArray[numpy.complex64,:],NDArray[numpy.complex64,:]])

    def test_correlate_8(self):
        dtype = numpy.float32
        self.run_test("def np_correlate_8(a,b):\n from numpy import correlate\n return correlate(a,b,'full')",
                  numpy.arange(7).astype(dtype) +  1j*numpy.arange(7).astype(dtype),
                  numpy.arange(12).astype(dtype) + 1j*numpy.arange(12).astype(dtype),
                  np_correlate_8=[NDArray[numpy.complex64,:],NDArray[numpy.complex64,:]])

    def test_correlate_9(self):
        dtype = numpy.float
        self.run_test("def np_correlate_9(a,b):\n from numpy import correlate\n return correlate(a,b,'full')",
                  numpy.arange(7).astype(dtype)  + 1j*numpy.arange(7).astype(dtype),
                  numpy.arange(12).astype(dtype) + 1j*numpy.arange(12).astype(dtype),
                  np_correlate_9=[NDArray[numpy.complex128,:],NDArray[numpy.complex128,:]])

    def test_correlate_10(self):
        self.run_test("def np_correlate_10(a,b):\n from numpy import correlate\n return correlate(a,b,'same')",
                  numpy.arange(12,dtype=float),
                  numpy.arange(7,dtype=numpy.float32),
                  np_correlate_10=[NDArray[float,:],NDArray[numpy.float32,:]])

    def test_correlate_11(self):
        self.run_test("def np_correlate_11(a,b):\n from numpy import correlate\n return correlate(a,b,'same')",
                  numpy.arange(12,dtype=numpy.float32),
                  numpy.arange(7,dtype=float),
                  np_correlate_11=[NDArray[numpy.float32,:],NDArray[float,:]])

    def test_convolve_1(self):
        self.run_test("def np_convolve_1(a,b):\n from numpy import convolve\n return convolve(a,b)",
                      numpy.arange(10,dtype=float),
                      numpy.arange(12,dtype=float),
                      np_convolve_1=[NDArray[float,:],NDArray[float,:]])
    def test_convolve_2(self):
        self.run_test("def np_convolve_2(a,b):\n from numpy import convolve\n return convolve(a,b)",
                  numpy.arange(12,dtype=float),
                  numpy.arange(10,dtype=float),
                  np_convolve_2=[NDArray[float,:],NDArray[float,:]])

    def test_convolve_3(self):
        self.run_test("def np_convolve_3(a,b):\n from numpy import convolve\n return convolve(a,b,'valid')",
                  numpy.arange(12,dtype=float),
                  numpy.arange(10,dtype=float),
                  np_convolve_3=[NDArray[float,:],NDArray[float,:]])

    def test_convolve_4(self):
        self.run_test("def np_convolve_4(a,b):\n from numpy import convolve\n return convolve(a,b,'same')",
                  numpy.arange(12,dtype=float),
                  numpy.arange(10,dtype=float),
                  np_convolve_4=[NDArray[float,:],NDArray[float,:]])

    def test_convolve_5(self):
        self.run_test("def np_convolve_5(a,b):\n from numpy import convolve\n return convolve(a,b,'same')",
                  numpy.arange(12,dtype=float),
                  numpy.arange(7,dtype=float),
                  np_convolve_5=[NDArray[float,:],NDArray[float,:]])

    def test_convolve_6(self):
        self.run_test("def np_convolve_6(a,b):\n from numpy import convolve\n return convolve(a,b,'full')",
                  numpy.arange(12.) + 1j*numpy.arange(12.),
                  numpy.arange(7.) + 1j* numpy.arange(7.),
                  np_convolve_6=[NDArray[complex,:],NDArray[complex,:]])

    def test_convolve_7(self):
        dtype = numpy.float32
        self.run_test("def np_convolve_7(a,b):\n from numpy import convolve\n return convolve(a,b,'full')",
                  numpy.arange(12).astype(dtype) + 1j*numpy.arange(12).astype(dtype),
                  numpy.arange(7).astype(dtype) +  1j*numpy.arange(7).astype(dtype),
                  np_convolve_7=[NDArray[numpy.complex64,:],NDArray[numpy.complex64,:]])

    def test_convolve_8(self):
        dtype = numpy.float32
        self.run_test("def np_convolve_8(a,b):\n from numpy import convolve\n return convolve(a,b,'full')",
                  numpy.arange(7).astype(dtype) + 1j*numpy.arange(7).astype(dtype),
                  numpy.arange(12).astype(dtype) +1j*numpy.arange(12).astype(dtype),
                  np_convolve_8=[NDArray[numpy.complex64,:],NDArray[numpy.complex64,:]])

    def test_convolve_9(self):
        dtype = numpy.float
        self.run_test("def np_convolve_9(a,b):\n from numpy import convolve\n return convolve(a,b,'full')",
                  numpy.arange(7).astype(dtype) + 1j*  numpy.arange(7).astype(dtype),
                  numpy.arange(12).astype(dtype) + 1j* numpy.arange(12).astype(dtype),
                  np_convolve_9=[NDArray[numpy.complex128,:],NDArray[numpy.complex128,:]])

    def test_convolve_10(self):
        self.run_test("def np_convolve_10(a,b):\n from numpy import convolve\n return convolve(a,b,'same')",
                  numpy.arange(12,dtype=float),
                  numpy.arange(7,dtype=numpy.float32),
                  np_convolve_10=[NDArray[float,:],NDArray[numpy.float32,:]])

    def test_convolve_11(self):
        self.run_test("def np_convolve_11(a,b):\n from numpy import convolve\n return convolve(a,b,'same')",
                  numpy.arange(12,dtype=numpy.float32),
                  numpy.arange(7,dtype=float),
                  np_convolve_11=[NDArray[numpy.float32,:],NDArray[float,:]])
        
    def test_copy0(self):
        code= '''
def test_copy0(x):
    import numpy as np
    y = x
    z = np.copy(x)
    x[0] = 10
    return x[0], y[0], z[0]'''
        self.run_test(code, numpy.array([1, 2, 3]), test_copy0=[NDArray[int,:]])

    def test_copy1(self):
        code="def test_copy1(n): import numpy as np ; r = np.ones((n,n)); g = np.copy(r); return g"
        self.run_test(code, 10, test_copy1=[int])

    def test_copy2(self):
        code="def test_copy2(n): import numpy as np ; r = np.ones((n,n)); g0 = np.copy(r); g1 = np.copy(r); g0[0] = 1 ; g1[0] = 2 ; return g0, g1"
        self.run_test(code, 10, test_copy2=[int])

    def test_copy3(self):
        code="def test_copy3(n): import numpy as np ; r = [[1]*n for _ in range(n)]; g = np.copy(r) ; return g"
        self.run_test(code, 10, test_copy3=[int])

    def test_copy4(self):
        code="def test_copy4(n): import numpy as np ; r = n; g = np.copy(r) ; return g"
        self.run_test(code, 10, test_copy4=[int])

    def test_copy5(self):
        code="def test_copy5(n): return n[:-1].copy()"
        self.run_test(code, numpy.array([1,2,3]), test_copy5=[NDArray[int,:]])

    def test_copy6(self):
        code="def test_copy6(n): return n[-1].copy()"
        self.run_test(code, numpy.array([[1],[2],]), test_copy6=[NDArray[int,:,:]])

    def test_clip0(self):
        self.run_test("def np_clip0(a): return a.clip(1,8)", numpy.arange(10), np_clip0=[NDArray[int,:]])

    def test_clip1(self):
        self.run_test("def np_clip1(a): from numpy import  clip ; return clip(a,3,6)", numpy.arange(10), np_clip1=[NDArray[int,:]])

    def test_concatenate0(self):
        self.run_test("def np_concatenate0(a): from numpy import array, concatenate ; b = array([[5, 6]]) ; return concatenate((a,b))", numpy.array([[1, 2], [3, 4]]), np_concatenate0=[NDArray[int,:,:]])

    def test_concatenate1(self):
        self.run_test("def np_concatenate1(a): from numpy import array, concatenate ; return concatenate([a,a])", numpy.array([[1, 2], [3, 4]]), np_concatenate1=[NDArray[int,:,:]])

    def test_concatenate2(self):
        self.run_test(
            "def np_concatenate2(a): from numpy import array, concatenate ; b = array([[5, 6]]).T ; return concatenate((a,b, b), axis=1)",
            numpy.array([[1, 2], [3, 4]]),
            np_concatenate2=[NDArray[int,:,:]])

    def test_concatenate3(self):
        self.run_test("def np_concatenate3(a): from numpy import array, concatenate ; return concatenate([[1],a + a])", numpy.array([1, 2]), np_concatenate3=[NDArray[int,:]])

    def test_hstack_empty(self):
        code = 'def np_test_stack_empty(a): import numpy as np;return np.stack(a)'
        with self.assertRaises(ValueError):
            self.run_test(code, [], np_test_stack_empty=[List[NDArray[float,:]]])

    def test_hstack0(self):
        self.run_test("def np_hstack0(a,b): import numpy as np; return np.hstack((a,b))",
                      numpy.array((1,2,3)),
                      numpy.array((2,3,4)),
                      np_hstack0=[NDArray[int,:],NDArray[int,:]])

    def test_hstack1(self):
        self.run_test("def np_hstack1(a,b): import numpy as np; return np.hstack((a+1,b))",
                      numpy.array(([1],[2],[3])),
                      numpy.array(([2],[3],[4])),
                      np_hstack1=[NDArray[int,:,:],NDArray[int,:,:]])

    def test_vstack0(self):
        self.run_test("def np_vstack0(a,b): import numpy as np; return np.vstack((a,b))",
                      numpy.array((1,2,3)),
                      numpy.array((2,3,4)),
                      np_vstack0=[NDArray[int,:],NDArray[int,:]])

    def test_vstack1(self):
        self.run_test("def np_vstack1(a,b): import numpy as np; return np.vstack((a,b+b))",
                      numpy.array(([1],[2],[3])),
                      numpy.array(([2],[3],[4])),
                      np_vstack1=[NDArray[int,:,:],NDArray[int,:,:]])

    def test_stack0(self):
        self.run_test("def np_stack0(A): import numpy as np; return np.stack(A,axis=0)",
                      [numpy.ones((2,2)), 2*numpy.ones((2,2))],
                      np_stack0=[List[NDArray[float,:,:],NDArray[float,:,:]]])

    def test_stack1(self):
        self.run_test("def np_stack1(A): import numpy as np; return np.stack(A,axis=1)",
                      [numpy.ones((2,2,2)), 2*numpy.ones((2,2,2))],
                      np_stack1=[List[NDArray[float,:,:,:],NDArray[float,:,:,:]]])

    def test_stack2(self):
        self.run_test("def np_stack2(a,b): import numpy as np; return np.stack([a,b],axis=0)",
                      numpy.ones((2,2)), 
                      2*numpy.ones((2,2)),
                      np_stack2=[NDArray[float,:,:],NDArray[float,:,:]])

    def test_stack3(self):
        self.run_test("def np_stack3(a,b): import numpy as np; return np.stack((a,b),axis=0)",
                      numpy.ones((2,2)), 
                      2*numpy.ones((2,2)),
                      np_stack3=[NDArray[float,:,:],NDArray[float,:,:]])

    def test_stack4(self):
        self.run_test("def np_stack4(a,b): import numpy as np; return np.stack((2 * a,b),axis=0)",
                      numpy.ones((4,2,3)), 
                      2*numpy.ones((4,2,3)),
                      np_stack4=[NDArray[float,:,:,:],NDArray[float,:,:,:]])

    def test_stack5(self):
        self.run_test("def np_stack5(a): import numpy as np; return np.stack((a[:,1:],a[:,1:]),axis=1)",
                      numpy.ones((4,3,3)),
                      np_stack5=[NDArray[float,:,:,:]])

    def test_bincount0(self):
        self.run_test("def np_bincount0(a): from numpy import bincount ; return bincount(a)", numpy.arange(5), np_bincount0=[NDArray[int,:]])

    def test_bincount1(self):
        self.run_test("def np_bincount1(a, w): from numpy import bincount; return bincount(a,w)", numpy.array([0, 1, 1, 2, 2, 2]), numpy.array([0.3, 0.5, 0.2, 0.7, 1., -0.6]), np_bincount1=[NDArray[int,:], NDArray[float,:]])
    def test_bincount2(self):
        self.run_test("def np_bincount2(a, w): from numpy import bincount; return bincount(a + 1,w)", numpy.array([0, 1, 1, 2, 2, 2]), numpy.array([0.3, 0.5, 0.2, 0.7, 1., -0.6]), np_bincount2=[NDArray[int,:], NDArray[float,:]])

    def test_bincount3(self):
        self.run_test("def np_bincount3(a): from numpy import bincount,ones; x=ones((a.shape[0]+2), dtype=int); return bincount(a, x[:-2])", numpy.array([0, 1, 1, 2, 2, 2]), np_bincount3=[NDArray[int,:]])

    def test_binary_repr0(self):
        self.run_test("def np_binary_repr0(a): from numpy import binary_repr ; return binary_repr(a)", 3, np_binary_repr0=[int])

    def test_binary_repr1(self):
        self.run_test("def np_binary_repr1(a): from numpy import binary_repr ; return binary_repr(a)", -3, np_binary_repr1=[int])

    def test_binary_repr2(self):
        self.run_test("def np_binary_repr2(a): from numpy import binary_repr ; return binary_repr(a,4)", 3, np_binary_repr2=[int])

    def test_binary_repr3(self):
        self.run_test("def np_binary_repr3(a): from numpy import binary_repr ; return binary_repr(a,4)", -3, np_binary_repr3=[int])

    def test_base_repr0(self):
        self.run_test("def np_base_repr0(a): from numpy import base_repr ; return base_repr(a)", 5, np_base_repr0=[int])

    def test_base_repr1(self):
        self.run_test("def np_base_repr1(a): from numpy import base_repr ; return base_repr(a,5)", 6, np_base_repr1=[int])

    def test_base_repr2(self):
        self.run_test("def np_base_repr2(a): from numpy import base_repr ; return base_repr(a,5,3)", 7, np_base_repr2=[int])

    def test_base_repr3(self):
        self.run_test("def np_base_repr3(a): from numpy import base_repr ; return base_repr(a, 16)", 10, np_base_repr3=[int])

    def test_base_repr4(self):
        self.run_test("def np_base_repr4(a): from numpy import base_repr ; return base_repr(a, 16)", 32, np_base_repr4=[int])

    def test_base_repr5(self):
        self.run_test("def np_base_repr5(a): from numpy import base_repr ; return base_repr(a)", -5, np_base_repr5=[int])

    def test_base_repr6(self):
        self.run_test("def np_base_repr6(a): from numpy import base_repr ; return base_repr(a)", 0, np_base_repr6=[int])

    def test_base_repr7(self):
        self.run_test("def np_base_repr7(a): from numpy import base_repr ; return base_repr(a,5)", 0, np_base_repr7=[int])

    def test_average0(self):
        self.run_test("def np_average0(a): from numpy import average ; return average(a)", numpy.arange(10), np_average0=[NDArray[int,:]])

    def test_average1(self):
        self.run_test("def np_average1(a): from numpy import average ; return average(a,1)", numpy.arange(10).reshape(2,5), np_average1=[NDArray[int,:,:]])

    def test_average2(self):
        self.run_test("def np_average2(a): from numpy import average ; return average(a,None, range(10))", numpy.arange(10), np_average2=[NDArray[int,:]])

    def test_average3(self):
        self.run_test("def np_average3(a): from numpy import average ; return average(a,None, a)", numpy.arange(10).reshape(2,5), np_average3=[NDArray[int,:,:]])

    def test_atleast_1d0(self):
        self.run_test("def np_atleast_1d0(a): from numpy import atleast_1d ; return atleast_1d(a)", 1, np_atleast_1d0=[int])

    def test_atleast_1d1(self):
        self.run_test("def np_atleast_1d1(a): from numpy import atleast_1d ; r = atleast_1d(a) ; return r is a", numpy.arange(2), np_atleast_1d1=[NDArray[int,:]])

    def test_atleast_2d0(self):
        self.run_test("def np_atleast_2d0(a): from numpy import atleast_2d ; return atleast_2d(a)", 1, np_atleast_2d0=[int])

    def test_atleast_2d1(self):
        self.run_test("def np_atleast_2d1(a): from numpy import atleast_2d ; r = atleast_2d(a) ; return r is a", numpy.arange(2).reshape(1,2), np_atleast_2d1=[NDArray[int,:,:]])

    def test_atleast_2d2(self):
        self.run_test("def np_atleast_2d2(a): from numpy import atleast_2d ; r = atleast_2d(a) ; return r", numpy.arange(2), np_atleast_2d2=[NDArray[int,:]])

    def test_atleast_3d0(self):
        self.run_test("def np_atleast_3d0(a): from numpy import atleast_3d ; return atleast_3d(a)", 1, np_atleast_3d0=[int])

    def test_atleast_3d1(self):
        self.run_test("def np_atleast_3d1(a): from numpy import atleast_3d ; r = atleast_3d(a) ; return r is a", numpy.arange(8).reshape(2,2,2), np_atleast_3d1=[NDArray[int,:,:,:]])

    def test_atleast_3d2(self):
        self.run_test("def np_atleast_3d2(a): from numpy import atleast_3d ; r = atleast_3d(a) ; return r", numpy.arange(8).reshape(2,4), np_atleast_3d2=[NDArray[int,:,:]])

    def test_atleast_3d3(self):
        self.run_test("def np_atleast_3d3(a): from numpy import atleast_3d ; r = atleast_3d(a) ; return r", numpy.arange(8), np_atleast_3d3=[NDArray[int,:]])

    def test_asscalar0(self):
        self.run_test("def np_asscalar0(a): from numpy import asscalar; return asscalar(a)", numpy.array([1], numpy.int32), np_asscalar0=[NDArray[numpy.int32,:]])

    def test_asscalar1(self):
        self.run_test("def np_asscalar1(a): from numpy import asscalar; return asscalar(a)", numpy.array([[1]], numpy.int64), np_asscalar1=[NDArray[numpy.int64,:,:]])

    def test_ascontiguousarray0(self):
        self.run_test("def np_ascontiguousarray0(a):\n from numpy import ascontiguousarray\n return ascontiguousarray(a)", (1,2,3), np_ascontiguousarray0=[Tuple[int, int, int]])

    def test_asarray_chkfinite0(self):
        self.run_test("def np_asarray_chkfinite0(a):\n from numpy import asarray_chkfinite\n return asarray_chkfinite(a)", (1,2,3), np_asarray_chkfinite0=[Tuple[int, int, int]])

    def test_asarray_chkfinite1(self):
        self.run_test("def np_asarray_chkfinite1(a, x):\n from numpy import asarray_chkfinite\n try: return asarray_chkfinite(a)\n except ValueError: return asarray_chkfinite(x)", [[1.,2.],[numpy.nan,4.]], [[1.,2.],[3.,4.]], np_asarray_chkfinite1=[List[List[float]], List[List[float]]])

    def test_asarray0(self):
        self.run_test("def np_asarray0(a):\n from numpy import asarray\n return asarray(a)", (1,2,3), np_asarray0=[Tuple[int, int, int]])

    def test_asarray1(self):
        self.run_test("def np_asarray1(a):\n from numpy import asarray\n return asarray(a)", [(1,2),(3,4)], np_asarray1=[List[Tuple[int, int]]])

    def test_asarray2(self):
        self.run_test("def np_asarray2(a):\n from numpy import asarray, int8\n return asarray(a, int8)", [1., 2., 3.], np_asarray2=[List[float]])

    def test_asarray3(self):
        self.run_test("def np_asarray3(a):\n from numpy import asarray; b = asarray(a) ; return a is b", numpy.arange(3), np_asarray3=[NDArray[int,:]])

    def test_asarray4(self):
        self.run_test("def np_asarray4(a):\n from numpy import asarray\n return asarray(a[1:])", [(1,2),(3,4)], np_asarray4=[List[Tuple[int, int]]])

    def test_asarray5(self):
        self.run_test("def np_asarray5(a):\n from numpy import asarray\n return asarray(a)", 1., np_asarray5=[float])

    def test_asarray6(self):
        self.run_test("def np_asarray6(a):\n from numpy import asarray\n return asarray(a, dtype=int)", 1.5, np_asarray6=[float])

    def test_asfarray0(self):
        self.run_test("def np_asfarray0(a):\n from numpy import asfarray; b = asfarray(a) ; return a is b", numpy.arange(3.), np_asfarray0=[NDArray[float,:]])

    def test_asfarray1(self):
        self.run_test("def np_asfarray1(a):\n from numpy import asfarray; b = asfarray(a) ; return a is not b", numpy.arange(3), np_asfarray1=[NDArray[int,:]])

    def test_astype0(self):
        self.run_test("def np_astype0(a):\n return a.astype(float)", numpy.arange(3), np_astype0=[NDArray[int,:]])

    def test_astype1(self):
        self.run_test("def np_astype1(a):\n import numpy as jumpy; return a.astype(jumpy.uint8)", numpy.arange(257), np_astype1=[NDArray[int,:]])

    def test_array_str0(self):
        self.run_test("def np_array_str0(x): from numpy import array_str ; return array_str(x)", numpy.arange(3), np_array_str0=[NDArray[int,:]])

    def test_array_split0(self):
        self.run_test("def np_array_split0(a): from numpy import array_split, array2string ; return list(map(array2string,array_split(a, 3)))", numpy.arange(12), np_array_split0=[NDArray[int,:]])

    def test_array_split1(self):
        self.run_test("def np_array_split1(a): from numpy import array_split, array2string ; return list(map(array2string,array_split(a, 5)))", numpy.arange(12), np_array_split1=[NDArray[int,:]])

    def test_array_split2(self):
        self.run_test("def np_array_split2(a): from numpy import array_split, array2string ; return list(map(array2string,array_split(a, 4)))", numpy.arange(12).reshape(6,2), np_array_split2=[NDArray[int,:,:]])

    def test_array_split3(self):
        self.run_test("def np_array_split3(a): from numpy import array_split, array2string ; return list(map(array2string,array_split(a, [0,1,5])))", numpy.arange(12).reshape(6,2), np_array_split3=[NDArray[int,:,:]])

    def test_array_equiv0(self):
        self.run_test("def np_array_equiv0(a): from numpy import array_equiv ;  b = [1,2] ; return array_equiv(a,b)", [1, 2], np_array_equiv0=[List[int]])

    def test_array_equiv1(self):
        self.run_test("def np_array_equiv1(a): from numpy import array_equiv ;  b = [1,3] ; return array_equiv(a,b)", [1, 2], np_array_equiv1=[List[int]])

    def test_array_equiv2(self):
        self.run_test("def np_array_equiv2(a): from numpy import array_equiv ;  b = [[1,2],[1,2]] ; return array_equiv(a,b)", [1, 2], np_array_equiv2=[List[int]])

    def test_array_equiv3(self):
        self.run_test("def np_array_equiv3(a): from numpy import array_equiv ;  b = [[1,2],[1,3]] ; return array_equiv(a,b)", [1, 2], np_array_equiv3=[List[int]])

    def test_array_equal0(self):
        self.run_test("def np_array_equal0(a): from numpy import array_equal ;  b = [1,2] ; return array_equal(a,b)", [1, 2], np_array_equal0=[List[int]])

    def test_array_equal1(self):
        self.run_test("def np_array_equal1(a): from numpy import array, array_equal ;  b = array([1,2], dtype=int) ; return array_equal(a,b)", numpy.array([1,2]), np_array_equal1=[NDArray[int,:]])

    def test_array_equal2(self):
        self.run_test("def np_array_equal2(a): from numpy import array, array_equal ;  b = array([[1,2],[3,5]]) ; return array_equal(a,b)", numpy.array([[1,2],[3,5]]), np_array_equal2=[NDArray[int,:,:]])

    def test_array_equal3(self):
        self.run_test("def np_array_equal3(a): from numpy import array, array_equal ;  b = array([[1,2],[4,5]]) ; return array_equal(a,b)", numpy.array([[1,2],[3,5]]), np_array_equal3=[NDArray[int,:,:]])

    def test_array_equal4(self):
        self.run_test("def np_array_equal4(a): from numpy import array, array_equal ;  b = array([1,2,3]) ; return array_equal(a,b)", numpy. array([1,2]), np_array_equal4=[NDArray[int,:]])

    def test_array2string0(self):
        self.run_test("def np_array2string0(x): from numpy import array2string ; return array2string(x)", numpy.arange(3), np_array2string0=[NDArray[int,:]])

    def test_argwhere0(self):
        self.run_test("def np_argwhere0(x): from numpy import argwhere ; return argwhere(x)", numpy.arange(6), np_argwhere0=[NDArray[int,:]])

    def test_argwhere1(self):
        self.run_test("def np_argwhere1(x): from numpy import argwhere ; return argwhere(x>8)", numpy.arange(6), np_argwhere1=[NDArray[int,:]])

    def test_argwhere2(self):
        self.run_test("def np_argwhere2(x): from numpy import argwhere ; return argwhere(x>0)", numpy.arange(6).reshape(2,3), np_argwhere2=[NDArray[int,:,:]])

    def test_around0(self):
        self.run_test("def np_around0(x): from numpy import around ; return around(x)", [0.37, 1.64], np_around0=[List[float]])

    def test_around1(self):
        self.run_test("def np_around1(x): from numpy import around ; return around(x, 1)", [0.37, 1.64], np_around1=[List[float]])

    def test_around2(self):
        """ Check rounding on the left side of comma. """
        self.run_test("""
            def np_around2(x):
                from numpy import around
                return around(x, -1)""", [37.4, 164.65], np_around2=[List[float]])

    def test_around3(self):
        self.run_test("def np_around3(x): from numpy import around ; return around(x)", [.5, 1.5, 2.5, 3.5, 4.5], np_around3=[List[float]])

    def test_around4(self):
        self.run_test("def np_around4(x): from numpy import around ; return around(x,1)", [1,2,3,11], np_around4=[List[int]])

    def test_around5(self):
        self.run_test("def np_around5(x): from numpy import around ; return around(x,-1)", [1,2,3,11], np_around5=[List[int]])

    def test_argsort0(self):
        self.run_test("def np_argsort0(x): from numpy import argsort ; return argsort(x)", numpy.array([3, 1, 2]), np_argsort0=[NDArray[int,:]])

    def test_argsort1(self):
        self.run_test("def np_argsort1(x): return x.argsort()", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort1=[NDArray[int,:,:]])

    def test_argmax0(self):
        self.run_test("def np_argmax0(a): return a.argmax()", numpy.arange(6).reshape(2,3), np_argmax0=[NDArray[int,:,:]])

    def test_argmax1(self):
        self.run_test("def np_argmax1(a): from numpy import argmax ; return argmax(a+a)", numpy.arange(6).reshape(2,3), np_argmax1=[NDArray[int,:,:]])

    def test_argmax2(self):
        self.run_test("def np_argmax2(a): from numpy import argmax ; return argmax(a, 0)", numpy.arange(30).reshape(2,3,5), np_argmax2=[NDArray[int,:,:,:]])

    def test_argmin0(self):
        self.run_test("def np_argmin0(a): return a.argmin()", numpy.arange(6).reshape(2,3), np_argmin0=[NDArray[int,:,:]])

    def test_argmin1(self):
        self.run_test("def np_argmin1(a): from numpy import argmin ; return argmin(a)", [1,2,3], np_argmin1=[List[int]])

    def test_argmin2(self):
        self.run_test("def np_argmin2(a): from numpy import argmin ; return argmin(a, 0)", numpy.arange(30).reshape(2,3,5), np_argmin2=[NDArray[int,:,:,:]])

    def test_argmin3(self):
        self.run_test("def np_argmin3(a): from numpy import argmin ; return argmin(a, 1)", numpy.arange(30).reshape(2,3,5), np_argmin3=[NDArray[int,:,:,:]])

    def test_argmin4(self):
        self.run_test("def np_argmin4(a): from numpy import argmin ; return argmin(a, 2)", numpy.arange(30).reshape(2,3,5), np_argmin4=[NDArray[int,:,:,:]])

    def test_argmin5(self):
        self.run_test("def np_argmin5(a): from numpy import argmin ; return argmin(a[None, :] + a[:, None], 0)", numpy.arange(30), np_argmin5=[NDArray[int,:]])

    def test_argmin6(self):
        self.run_test("def np_argmin6(a): from numpy import argmin ; return argmin(a[None, :] + a[:, None], 1)", numpy.arange(30), np_argmin6=[NDArray[int,:]])

    def test_append0(self):
        self.run_test("def np_append0(a): from numpy import append ; b = [[4, 5, 6], [7, 8, 9]] ; return append(a,b)", [1, 2, 3], np_append0=[List[int]])

    def test_append1(self):
        self.run_test("def np_append1(a): from numpy import append,array ; b = array([[4, 5, 6], [7, 8, 9]]) ; return append(a,b)", [1, 2, 3], np_append1=[List[int]])

    def test_append2(self):
        self.run_test("def np_append2(a): from numpy import append,array ; b = array([[4, 5, 6], [7, 8, 9]]) ; return append(a,b)", numpy.array([1, 2, 3]), np_append2=[NDArray[int,:]])

    def test_append3(self):
        self.run_test("def np_append3(a): from numpy import append,array ; return append(a, 1)", numpy.array([1, 2, 3]), np_append3=[NDArray[int,:]])

    def test_append4(self):
        self.run_test("def np_append4(a): from numpy import append ; b = 4 ; return append(a,b)", [1, 2, 3], np_append4=[List[int]])

    def test_append5(self):
        self.run_test("def np_append5(a): from numpy import append,array ; return append(a, 1)", numpy.array([[1], [2], [3]]), np_append5=[NDArray[int,:, :]])

    def test_append6(self):
        self.run_test("def np_append6(a): from numpy import append,array ; b = array([[4, 5, 6], [7, 8, 9]]) ; return append(a.T, b)", numpy.array([[1], [2], [3]]), np_append6=[NDArray[int,:, :]])

    def test_append7(self):
        self.run_test("def np_append7(a): from numpy import append,array ; b = array([[4, 5, 6], [7, 8, 9]]) ; return append(a[0], b)", numpy.array([[1], [2], [3]]), np_append7=[NDArray[int,:, :]])

    def test_append8(self):
        self.run_test("def np_append8(a): from numpy import append,array ; b = array([[4, 5, 6], [7, 8, 9]]) ; return append(a[:], b)", numpy.array([[1], [2], [3]]), np_append8=[NDArray[int,:, :]])

    def test_angle0(self):
        self.run_test("def np_angle0(a): from numpy import angle ; return angle(a)", [1.0+0j, 1.0j, 1+1j, -1.0+0j, -1.0j, -1-1j], np_angle0=[List[complex]])

    def test_angle1(self):
                      self.run_test("def np_angle1(a): from numpy import angle ; return angle(a)", numpy.array([1.0+0j, 1.0j, 1+1j, -1.0+0j, -1.0j, -1-1j]), np_angle1=[NDArray[complex,:]])

    def test_angle2(self):
        self.run_test("def np_angle2(a): from numpy import angle ; return angle(a,True)", 1 + 1j, np_angle2=[complex])

    def test_angle3(self):
        self.run_test("def np_angle3(a): from numpy import angle ; return angle(a,True)", 1, np_angle3=[int])

    def test_any0(self):
        self.run_test("def np_any0(a): from numpy import any ; return any(a)", numpy.array([[True, False], [True, True]]), np_any0=[NDArray[bool,:,:]])

    def test_any1(self):
        self.run_test("def np_any1(a): from numpy import any ;  return any(a, 0)", numpy.array([[True, False], [False, False]]), np_any1=[NDArray[bool,:,:]])

    def test_any2(self):
        self.run_test("def np_any2(a): from numpy import any ; return any(a)", [-1, 0, 5], np_any2=[List[int]])

    def test_any3(self):
        self.run_test("def np_any3(a): from numpy import any ; return any(a).any(0)", [-1, 0, 5], np_any3=[List[int]])

    def test_any4(self):
        self.run_test("def np_any4(a): from numpy import any ; return any(a > 10)", numpy.array([-1, 0, 5]), np_any4=[NDArray[int, :]])

    def test_any5(self):
        self.run_test("def np_any5(a): from numpy import any ; return any(a > 10.0)", numpy.array([-1.0, 0.0, 5.0]), np_any5=[NDArray[float,:]])

    def test_any6(self):
        self.run_test("def np_any6(a): from numpy import any ; return any(a > 10.0)", numpy.array([[-1.0, 0.0], [5.0, 3.0]]), np_any6=[NDArray[float,:,:]])

    def test_any7(self):
        self.run_test("def np_any7(a): from numpy import any ; return any(a)", numpy.array([[False, False], [False, False]]), np_any7=[NDArray[bool,:,:]])

    def test_array1D_(self):
        self.run_test("def np_array1D_(a):\n from numpy import array\n return array(a)", [1,2,3], np_array1D_=[List[int]])

    def test_array2D_(self):
        self.run_test("def np_array2D_(a):\n from numpy import array\n return array(a)", [[1,2],[3,4]], np_array2D_=[List[List[int]]])

    def test_array_typed(self):
        self.run_test("def np_array_typed(a):\n from numpy import array, int64\n return array(a, int64)", [1.,2.,3.], np_array_typed=[List[float]])

    def test_zeros_(self):
        self.run_test("def np_zeros_(a): from numpy import zeros; return zeros(a)", (10, 5), np_zeros_=[Tuple[int, int]])

    def test_ones_(self):
        self.run_test("def np_ones_(a): from numpy import ones; return ones(a)", (10, 5), np_ones_=[Tuple[int, int]])

    def test_full_0(self):
        self.run_test("def np_full_0(a): from numpy import full; return full(a, 1.5)", (10, 5), np_full_0=[Tuple[int, int]])

    def test_full_1(self):
        self.run_test("def np_full_1(a): from numpy import full; return full(a, 1.5, dtype=int)", (10, 5), np_full_1=[Tuple[int, int]])

    def test_full_2(self):
        self.run_test("def np_full_2(a): from numpy import full; return full(a, 1.5)", 10, np_full_2=[int])

    def test_full_3(self):
        self.run_test("def np_full_3(a): from numpy import full; return full(a, 1.5, dtype=int)", 10, np_full_3=[int])

    def test_flat_zeros_(self):
        self.run_test("def np_flat_zeros_(a): from numpy import zeros; return zeros(a)", 10, np_flat_zeros_=[int])

    def test_flat_ones_(self):
        self.run_test("def np_flat_ones_(a): from numpy import ones; return ones(a)", 5, np_flat_ones_=[int])

    def test_input_array_(self):
        self.run_test("import numpy\n\ndef input_array_(a):\n return a.shape", numpy.array([[1,2],[3,4]]), input_array_=[NDArray[int,:,:]])

    def test_change_array1D_(self):
        """ Assign to lowest dimension of an array. """
        self.run_test("""
            def np_change_array1D_(a):
                a[0, 0, 0] = 36
                return a""",
                      numpy.array([[[1, 2], [3, 4]], [[5, 6], [7, 8]]]),
                      np_change_array1D_=[NDArray[int,:,:,:]])

    def test_str_(self):
        self.run_test("def np_str_(a): return str(a)", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_str_=[NDArray[int,:,:,:]])

    def test_len_(self):
        self.run_test("def np_len_(a): return len(a)", numpy.array([[[1,2],[3,4]],[[5,6],[7,8]]]), np_len_=[NDArray[int,:,:,:]])

    def test_empty_(self):
        self.run_test("def np_empty_(a):\n from numpy import empty\n a = empty(a)\n return a.strides, len(a)", (3, 2), np_empty_=[Tuple[int, int]])

    def test_empty_uint_shape(self):
        self.run_test("def np_empty_uint_shape(a):\n from numpy import empty, uint8\n a = empty((uint8(a), uint8(a)))\n return a.strides, len(a)", 3, np_empty_uint_shape=[int])

    def test_empty_integral_shape(self):
        self.run_test("def np_empty_integral_shape(n):\n from numpy import empty, uint8\n a = empty(5)\n return a.strides, len(a), n", 3, np_empty_integral_shape=[int])

    def test_ones_uint_shape(self):
        self.run_test("def np_ones_uint_shape(a):\n from numpy import ones, uint32\n a = ones((uint32(a), uint32(a)))\n return a.strides, len(a)", 3, np_ones_uint_shape=[int])

    def test_ones_integral_shape(self):
        self.run_test("def np_ones_integral_shape(n):\n from numpy import ones, uint8\n a = ones(5)\n return a.strides, len(a), n", 3, np_ones_integral_shape=[int])

    def test_zeros_uint_shape(self):
        self.run_test("def np_zeros_uint_shape(a):\n from numpy import zeros, int32\n a = zeros((int32(a), int32(a)))\n return a.strides, len(a)", 3, np_zeros_uint_shape=[int])

    def test_zeros_integral_shape(self):
        self.run_test("def np_zeros_integral_shape(n):\n from numpy import zeros, uint8\n a = zeros(5)\n return a.strides, len(a), n", 3, np_zeros_integral_shape=[int])

    def test_empty_kwargs(self):
        self.run_test("def np_empty_kwargs(a):\n from numpy import empty\n a = empty(a, dtype=int)\n return a.strides, len(a)", (3, 2), np_empty_kwargs=[Tuple[int, int]])

    def test_empty_kwargs2(self):
        self.run_test("def np_empty_kwargs2(a):\n from numpy import empty \n a = empty(a, dtype=float)\n return a.strides, a.shape", 3, np_empty_kwargs2=[int])

    def test_arange(self):
        self.run_test("def np_arange_(a):\n from numpy import arange\n return arange(a, dtype=float)", 10, np_arange_=[int])

    def test_arange1(self):
        self.run_test("def np_arange1_(a):\n from numpy import arange\n return arange(a, 10)", 1, np_arange1_=[int])

    def test_arange2(self):
        self.run_test("def np_arange2_(a):\n from numpy import arange\n return arange(a, 10)", 0.5, np_arange2_=[float])

    def test_arange3(self):
        self.run_test("def np_arange3_(a):\n from numpy import arange\n return arange(a, 9.5)", 0.5, np_arange3_=[float])

    def test_arange4(self):
        self.run_test("def np_arange4_(a):\n from numpy import arange\n return arange(a, 9.3, 1)", 0.2, np_arange4_=[float])

    def test_arange5(self):
        self.run_test("def np_arange5_(a):\n from numpy import arange\n return arange(a, 2, 0.3)", 1, np_arange5_=[int])

    def test_arange6(self):
        self.run_test("def np_arange6_(a):\n from numpy import arange\n return arange(a, 3.3, 0.5)", 0.2, np_arange6_=[float])

    def test_arange7(self):
        self.run_test("def np_arange7_(a):\n from numpy import arange\n return arange(a, 4.5, -0.2)", 1, np_arange7_=[int])

    def test_arange8(self):
        self.run_test("def np_arange8_(a):\n from numpy import arange\n return arange(a, 1, -0.1)", 4.5, np_arange8_=[float])

    def test_arange9(self):
        self.run_test("def np_arange9_(a):\n from numpy import arange\n return arange(a, -12, -3.2)", 4.5, np_arange9_=[float])

    def test_arange10(self):
        self.run_test("def np_arange10_(a):\n from numpy import arange\n return arange(a, -5.5, -0.1)", -5, np_arange10_=[int])

    def test_arange11(self):
        self.run_test("def np_arange11_(a):\n from numpy import arange, uint8\n return arange(a, 255, 1, uint8)", 0, np_arange11_=[int])

    def test_arange12(self):
        self.run_test("def np_arange12_(a):\n from numpy import arange, float32\n return arange(a, 25, 1., float32)", 0, np_arange12_=[int])

    def test_arange13(self):
        self.run_test("def np_arange12_(a):\n from numpy import arange, float32\n return arange(a, 25, dtype=float32)", 0, np_arange12_=[int])

    def test_arange14(self):
        self.run_test("def np_arange14_(a):\n from numpy import arange, float32\n return 50000 * arange(a, 25, dtype=float32)", 0, np_arange14_=[int])

    def test_arange15(self):
        self.run_test("def np_arange15_(a):\n from numpy import arange\n return arange(-4 * a, 1, 4)", 1, np_arange15_=[int])

    def test_arange16(self):
        self.run_test("def np_arange16_(a):\n from numpy import arange\n return arange(4 * a, -1, -4)", 1, np_arange16_=[int])

    def test_arange17(self):
        self.run_test("def np_arange17_(a):\n from numpy import arange\n return arange(4 * a, -1, -4)[:,None]", 1, np_arange17_=[int])

    def test_linspace(self):
        self.run_test("def np_linspace_(a):\n from numpy import linspace\n return linspace(a,4,32)", 1, np_linspace_=[int])

    def test_linspace1(self):
        self.run_test("def np_linspace1_(a):\n from numpy import linspace\n return linspace(a,32.5,2)", 0.4, np_linspace1_=[float])

    def test_linspace2(self):
        self.run_test("def np_linspace2_(a):\n from numpy import linspace\n return linspace(a,32.5,32, False)", 0.4, np_linspace2_=[float])

    def test_linspace3(self):
        self.run_test("def np_linspace3_(a):\n from numpy import linspace\n return linspace(1,a)", 4, np_linspace3_=[int])

    def test_linspace4(self):
        self.run_test("def np_linspace4_(a):\n from numpy import linspace\n return linspace(1,a,dtype=int)", 4, np_linspace4_=[int])

    def test_sin(self):
        self.run_test("def np_sin_(a):\n from numpy import sin\n return sin(a)", numpy.linspace(0,6), np_sin_=[NDArray[float,:]])

    def test_pi(self):
        self.run_test("def np_pi_():\n from numpy import pi\n return pi", np_pi_=[])

    def test_e(self):
        self.run_test("def np_e_():\n from numpy import e\n return e", np_e_=[])

    def test_ones_like_(self):
        self.run_test("def np_ones_like_(a):\n from numpy import ones_like, array\n return ones_like(array(a))", [[i,j,k,l] for i in range(5) for j in range(4) for k in range(6) for l in range(8)], np_ones_like_=[List[List[int]]])

    def test_full_like_0(self):
        self.run_test("def np_full_like_0(a):\n from numpy import full_like, array\n return full_like(array(a), fill_value=12.5)", [[i,j,k,l] for i in range(5) for j in range(4) for k in range(6) for l in range(8)], np_full_like_0=[List[List[int]]])

    def test_full_like_1(self):
        self.run_test("def np_full_like_1(a):\n from numpy import full_like, array\n return full_like(array(a), fill_value=12.5, dtype=float)", [[i,j,k,l] for i in range(5) for j in range(4) for k in range(6) for l in range(8)], np_full_like_1=[List[List[int]]])

    def test_zeros_like_(self):
        self.run_test("def np_zeros_like_(a):\n from numpy import zeros_like, array\n return zeros_like(array(a))", [[i,j,k,l] for i in range(5) for j in range(4) for k in range(6) for l in range(8)], np_zeros_like_=[List[List[int]]])

    def test_empty_like_1(self):
        """ Check empty_like numpy function without specified dtype. """
        code = """
            def np_empty_like_1(a):
                from numpy import empty_like
                b = empty_like(a)
                for i in range(2):
                    for j in range(3):
                        for k in range(4):
                            b[i, j, k] = i + j + k
                return b"""
        self.run_test(code,
                      numpy.arange(2 * 3 * 4).reshape(2, 3, 4),
                      np_empty_like_1=[NDArray[int,:,:,:]])

    def test_empty_like_2(self):
        """ Check empty_like numpy function with specific dtype argument. """
        code = """
            def np_empty_like_2(a):
                from numpy import empty_like
                b = empty_like(a, dtype=float)
                for i in range(2):
                    for j in range(3):
                        for k in range(4):
                            b[i, j, k] = i + j + k
                return b"""
        self.run_test(code,
                      numpy.arange(2 * 3 * 4).reshape(2, 3, 4),
                      np_empty_like_2=[NDArray[int,:,:,:]])

    def test_reshape0(self):
        self.run_test("def np_reshape0(a): return a.reshape((2,5))", numpy.arange(10), np_reshape0=[NDArray[int,:]])

    def test_reshape1(self):
        """Check reshaping with -1 argument."""
        code = """
def np_reshape1(a):
    return a.reshape((2,-1))"""
        self.run_test(code, numpy.arange(10), np_reshape1=[NDArray[int,:]])

    def test_reshape2(self):
        self.run_test("def np_reshape2(a): return a.reshape(5, 2)", numpy.arange(10), np_reshape2=[NDArray[int,:]])

    def test_reshape3(self):
        self.run_test("def np_reshape3(a): return (1 + a.reshape(1, a.size)), (1 + a[None])", numpy.arange(10), np_reshape3=[NDArray[int,:]])
        
    def test_reshape4(self):
        self.run_test("def np_reshape4(a): return (1 + a.reshape(5, -1)), (1 + a[None])", numpy.arange(10), np_reshape4=[NDArray[int,:]])
        
    def test_reshape5(self):
        self.run_test("def np_reshape5(a): return a.reshape(-1)", numpy.random.random((10,2)), np_reshape5=[NDArray[float,:,:]])
        
    def test_reshape6(self):
        code = "def test_reshape6(a): return a.reshape((-1,-1))"
        with self.assertRaises(ValueError):
            self.run_test(code, numpy.random.random((10,2)), test_reshape6=[NDArray[float,:,:]])

    def test_reshape7(self):
        self.run_test("def np_reshape7(a): return a.reshape(-10)", numpy.random.random((10,2)), np_reshape7=[NDArray[float,:,:]])

    def test_expand_dims1(self):
        code = """
import numpy
def np_expand_dims1(a,axis):
    return numpy.expand_dims(a,axis)"""
        self.run_test(code, numpy.arange(10,dtype=float),0, np_expand_dims1=[NDArray[float,:],int])
        self.run_test(code, numpy.arange(10,dtype=float),1, np_expand_dims1=[NDArray[float,:],int])
        
    def test_expand_dims2(self):
        code = """
import numpy
def np_expand_dims2(a,axis):
    return numpy.expand_dims(a,axis)"""
        self.run_test(code, numpy.random.randn(10,20),0, np_expand_dims2=[NDArray[float,:,:],int])
        self.run_test(code, numpy.random.randn(10,20),1, np_expand_dims2=[NDArray[float,:,:],int])
        self.run_test(code, numpy.random.randn(10,20),2, np_expand_dims2=[NDArray[float,:,:],int])
        
    def test_expand_dims3(self):
        code = """
import numpy
def np_expand_dims3(a,axis):
    return numpy.expand_dims(a,axis)"""
        self.run_test(code, numpy.random.randn(10,20,30),0, np_expand_dims3=[NDArray[float,:,:,:],int])
        self.run_test(code, numpy.random.randn(10,20,30),1, np_expand_dims3=[NDArray[float,:,:,:],int])
        self.run_test(code, numpy.random.randn(10,20,30),2, np_expand_dims3=[NDArray[float,:,:,:],int])
        self.run_test(code, numpy.random.randn(10,20,30),3, np_expand_dims3=[NDArray[float,:,:,:],int])
        

    def test_expand_dims4(self):
        code = '''

import numpy as np

def test1(x):
    y = np.expand_dims(x, -1)
    return y

def expand_dims4(x):
    A = np.array([-1.11312199, -0.99629629])
    return test1(x - A)'''
        self.run_test(code, numpy.random.randn(4,3,2), expand_dims4=[NDArray[float,:,:,:]])

    def test_duplicate(self):
        """Check array forwarded twice doesn't double free. """
        code = """
def np_duplicate(a):
    return a, a"""
        self.run_test(code, numpy.arange(10), np_duplicate=[NDArray[int,:]])

    def test_broadcast(self):
        """Check that ndarray returned twice doesn't double free. """
        code = """
def np_broadcast():
    import numpy
    a = numpy.arange(3)
    return a, a"""
        self.run_test(code, np_broadcast=[])

    def test_broadcast_slice(self):
        """Check that slicing an expression involving a broadcast works."""
        code = """
            def np_broadcast_slice(n):
                import numpy
                a = numpy.arange(n).reshape(2, n//2)
                return (a + 1)[:,1:]"""
        self.run_test(code, 20, np_broadcast_slice=[int])

    def test_broadcast_slice_again(self):
        """Check that slicing an expression involving a broadcast works."""
        code = """
            def np_broadcast_slice_again(n):
                import numpy
                a = numpy.arange(n).reshape(2, n//2)
                b = numpy.arange(n//2)
                return (a + b)[:,1:]"""
        self.run_test(code, 20, np_broadcast_slice_again=[int])

    def test_broadcast_dup(self):
        """Check that ndarray returned twice doesn't double free (reshaping)."""
        code = """
def np_broadcast_dup():
    import numpy
    a = numpy.arange(10)
    return a, a.reshape((2,5))"""
        self.run_test(code, np_broadcast_dup=[])

    def test_reshape_expr(self):
        self.run_test("def np_reshape_expr(a): return (a + a).reshape((2,5))", numpy.ones(10), np_reshape_expr=[NDArray[float,:]])

    def test_cumsum_(self):
        self.run_test("def np_cumsum_(a): return a.cumsum()", numpy.arange(10), np_cumsum_=[NDArray[int,:]])

    def test_cumsum2_(self):
        self.run_test("def np_cumsum2_(a): return a.cumsum()", numpy.arange(10).reshape(2,5), np_cumsum2_=[NDArray[int,:,:]])

    def test_cumsum3_(self):
        self.run_test("def np_cumsum3_(a): return a.cumsum(1)", numpy.arange(10).reshape(2,5), np_cumsum3_=[NDArray[int,:,:]])

    def test_cumsum4_(self):
        self.run_test("def np_cumsum4_(a): return a.cumsum(0)", numpy.arange(10).reshape(2,5), np_cumsum4_=[NDArray[int,:,:]])

    def test_cumsum5_(self):
        self.run_test("def np_cumsum5_(a): return a.cumsum(0)", numpy.arange(10), np_cumsum5_=[NDArray[int,:]])

    def test_sum_(self):
        self.run_test("def np_sum_(a): return a.sum()", numpy.arange(10), np_sum_=[NDArray[int,:]])

    def test_sum_of_bool(self):
        self.run_test("def np_sum_of_bool(a): return (a > 2).sum()", numpy.arange(10), np_sum_of_bool=[NDArray[int,:]])

    def test_sum_of_bool2(self):
        self.run_test("def np_sum_of_bool2(a): return (a > 2).sum(axis=1)", numpy.arange(10).reshape(5,2), np_sum_of_bool2=[NDArray[int,:,:]])

    def test_sum_scalar0(self):
        self.run_test("def np_sum_scalar0(a): return a.sum().sum()", numpy.arange(10), np_sum_scalar0=[NDArray[int,:]])

    def test_sum_scalar1(self):
        self.run_test("def np_sum_scalar1(a): return a.sum().sum(0)", numpy.arange(10), np_sum_scalar1=[NDArray[int,:]])

    def test_sum_neg_shape(self):
        self.run_test("def np_sum_neg_shape(a): return a.sum(axis=-1)", numpy.arange(10).reshape(5,2), np_sum_neg_shape=[NDArray[int,:,:]])

    def test_matrix_power0(self):
        self.run_test(
            "def np_matrix_power0(a): from numpy.linalg import matrix_power; return matrix_power(a, 0)",
            numpy.ones((10,10)),
            np_matrix_power0=[NDArray[float,:,:]]
        )

    def test_matrix_power1(self):
        self.run_test(
            "def np_matrix_power1(a): from numpy.linalg import matrix_power; return matrix_power(a, 1)",
            numpy.ones((10,10)),
            np_matrix_power1=[NDArray[float,:,:]]
        )

    def test_matrix_power2(self):
        self.run_test(
            "def np_matrix_power2(a): from numpy.linalg import matrix_power; return matrix_power(a, 5)",
            numpy.ones((10,10)),
            np_matrix_power2=[NDArray[float,:,:]]
        )




def omp_sections_private():
    sum = 7
    sum0 = 0

    if 'omp parallel':
        if 'omp sections private(sum0)':
            if 'omp section':
                sum0 = 0
                for i in range(0, 400):
                    sum0 += i
                if 'omp critical':
                    sum += sum0
            if 'omp section':
                sum0 = 0
                for i in range(400, 700):
                    sum0 += i
                if 'omp critical':
                    sum += sum0
            if 'omp section':
                sum0 = 0
                for i in range(700, 1000):
                    sum0 += i
                if 'omp critical':
                    sum += sum0

    known_sum = (999 * 1000) / 2 + 7;
    return known_sum == sum



# pythran export clip(complex128[], float64), limit (complex128[],float?)
# runas import numpy as np ; a = np.arange(2, dtype=complex); clip(a, .5), clip(a[2::4], .5)

import numpy as np


def limit (x, epsilon=1e-6):
    out = np.empty(shape=x.shape, dtype=x.dtype)
    mask1 = np.abs(x) < epsilon
    out[mask1] = 0
    mask2 = np.logical_not(mask1)
    out[mask2] = x[mask2] / np.abs(x[mask2])
    return out

def clip (z, _max):
    mask = np.abs(z) > _max
    z[mask] = limit(z[mask]) * _max
    return z



#from http://rosettacode.org/wiki/Real_constants_and_functions#Python
#pythran export test()
#runas test()

def test():
    import math
    x = 3.5
    y = -0.2
    print(math.e)          # e
    print(math.pi)         # pi
    print(math.sqrt(x))    # square root  (Also commonly seen as x ** 0.5 to obviate importing the math module)
    print(math.log(x))     # natural logarithm
    print(math.log10(x))   # base 10 logarithm
    print(math.exp(x))     # e raised to the power of x
    print(abs(x))          # absolute value
    print(math.floor(x))   # floor
    print(math.ceil(x))    # ceiling
    print(x ** y)          # exponentiation 
    print(pow(x, y))



def omp_parallel_sections_reduction():
    import math
    dt = 0.5
    rounding_error = 1.E-9

    sum = 7
    dsum = 0
    dt = 1. / 3.
    result = True
    product = 1
    logic_and = 1
    logic_or = 0
    bit_and = 1
    bit_or = 0
    i = 0
    exclusiv_bit_or = 0

    known_sum = (1000 * 999) / 2 + 7

    if 'omp parallel sections private(i) reduction(+:sum)':
        if 'omp section':
            for i in range(1,300):
                sum += i
        if 'omp section':
            for i in range(300,700):
                sum += i
        if 'omp section':
            for i in range(700,1000):
                sum += i

    if known_sum != sum:
        print("E: reduction(+:sum)")
        result = False

    diff = (1000 * 999) / 2

    if 'omp parallel sections private(i) reduction(-:diff)':
        if 'omp section':
            for i in range(1,300):
                diff -= i
        if 'omp section':
            for i in range(300,700):
                diff -= i
        if 'omp section':
            for i in range(700,1000):
                diff -= i

    if diff != 0:
        print("E: reduction(-:diff)")
        result = False

    dsum = 0
    dpt = 0
    for i in range(0, 20):
        dpt *= dt
    dknown_sum = (1 - dpt) / (1 - dt)

    if 'omp parallel sections private(i) reduction(+:dsum)':
        if 'omp section':
            for i in range(0,7):
                dsum += math.pow(dt, i)
        if 'omp section':
            for i in range(7,14):
                dsum += math.pow(dt, i)
        if 'omp section':
            for i in range(14,20):
                dsum += math.pow(dt, i)

    if abs(dsum-dknown_sum) > rounding_error:
        print("E: reduction(+:dsum)")
        result = False

    dsum = 0
    dpt = 0
    for i in range(0, 20):
        dpt *= dt
    ddiff = (1 - dpt) / (1 - dt)

    if 'omp parallel sections private(i) reduction(-:ddiff)':
        if 'omp section':
            for i in range(0,6):
                ddiff -= math.pow(dt, i)
        if 'omp section':
            for i in range(6,12):
                ddiff -= math.pow(dt, i)
        if 'omp section':
            for i in range(12,20):
                ddiff -= math.pow(dt, i)

    if abs(ddiff) > rounding_error:
        print("E: reduction(-:ddiff)")
        result = False

    if 'omp parallel sections private(i) reduction(*:product)':
        if 'omp section':
            for i in range(1,3):
                product *= i
        if 'omp section':
            for i in range(3,6):
                product *= i
        if 'omp section':
            for i in range(6,11):
                product *= i

    known_product = 3628800

    if known_product != product:
        print("E: reduction(*:product)")
        result = False

    logics = [1 for i in range(0,1000)]

    if 'omp parallel sections private(i) reduction(&&:logic_and)':
        if 'omp section':
            for i in range(0, 300):
                logic_and = (logic_and and logics[i])
        if 'omp section':
            for i in range(300, 700):
                logic_and = (logic_and and logics[i])
        if 'omp section':
            for i in range(700, 1000):
                logic_and = (logic_and and logics[i])

    if not logic_and:
        print("E: reduction(&&:logic_and)")
        result = False

    logic_and = 1;
    logics[1000//2]=0

    if 'omp parallel sections private(i) reduction(&&:logic_and)':
        if 'omp section':
            for i in range(0, 300):
                logic_and = (logic_and and logics[i])
        if 'omp section':
            for i in range(300, 700):
                logic_and = (logic_and and logics[i])
        if 'omp section':
            for i in range(700, 1000):
                logic_and = (logic_and and logics[i])

    if logic_and:
        print("E: reduction(&&:logic_and) with logics[1000/2]=0")
        result = False

    logics = [0 for i in range(0,1000)]

    if 'omp parallel sections private(i) reduction(||:logic_or)':
        if 'omp section':
            for i in range(0, 300):
                logic_or = (logic_or or logics[i])
        if 'omp section':
            for i in range(300, 700):
                logic_or = (logic_or or logics[i])
        if 'omp section':
            for i in range(700, 1000):
                logic_or = (logic_or or logics[i])

    if logic_or:
        print("E: reduction(||:logic_or)")
        result = False

    logic_or = 0;
    logics[1000//2]=1

    if 'omp parallel sections private(i) reduction(||:logic_or)':
        if 'omp section':
            for i in range(0, 300):
                logic_or = (logic_or or logics[i])
        if 'omp section':
            for i in range(300, 700):
                logic_or = (logic_or or logics[i])
        if 'omp section':
            for i in range(700, 1000):
                logic_or = (logic_or or logics[i])

    if not logic_or:
        print("E: reduction(||:logic_or) with logics[1000/2]=1")
        result = False

    logics = [1 for i in range(0,1000)]

    if 'omp parallel sections private(i) reduction(&:bit_and)':
        if 'omp section':
            for i in range(0, 300):
                bit_and = (bit_and & logics[i])
        if 'omp section':
            for i in range(300, 700):
                bit_and = (bit_and & logics[i])
        if 'omp section':
            for i in range(700, 1000):
                bit_and = (bit_and & logics[i])

    if not bit_and:
        print("E: reduction(&:bit_and)")
        result = False

    bit_and = 1;
    logics[1000//2]=0

    if 'omp parallel sections private(i) reduction(&:bit_and)':
        if 'omp section':
            for i in range(0, 300):
                bit_and = (bit_and & logics[i])
        if 'omp section':
            for i in range(300, 700):
                bit_and = (bit_and & logics[i])
        if 'omp section':
            for i in range(700, 1000):
                bit_and = (bit_and & logics[i])

    if bit_and:
        print("E: reduction(&:bit_and) with logics[1000/2]=0")
        result = False

    logics = [0 for i in range(0,1000)]

    if 'omp parallel sections private(i) reduction(|:bit_or)':
        if 'omp section':
            for i in range(0, 300):
                bit_or = (bit_or | logics[i])
        if 'omp section':
            for i in range(300, 700):
                bit_or = (bit_or | logics[i])
        if 'omp section':
            for i in range(700, 1000):
                bit_or = (bit_or | logics[i])

    if bit_or:
        print("E: reduction(|:bit_or)")
        result = False

    bit_or = 0;
    logics[1000//2]=1

    if 'omp parallel sections private(i) reduction(|:bit_or)':
        if 'omp section':
            for i in range(0, 300):
                bit_or = (bit_or | logics[i])
        if 'omp section':
            for i in range(300, 700):
                bit_or = (bit_or | logics[i])
        if 'omp section':
            for i in range(700, 1000):
                bit_or = (bit_or | logics[i])

    if not bit_or:
        print("E: reduction(|:bit_or) with logics[1000/2]=1")
        result = False

    logics = [0 for i in range(0,1000)]

    if 'omp parallel sections private(i) reduction(^:exclusiv_bit_or)':
        if 'omp section':
            for i in range(0, 300):
                exclusiv_bit_or = (exclusiv_bit_or ^ logics[i])
        if 'omp section':
            for i in range(300, 700):
                exclusiv_bit_or = (exclusiv_bit_or ^ logics[i])
        if 'omp section':
            for i in range(700, 1000):
                exclusiv_bit_or = (exclusiv_bit_or ^ logics[i])

    if exclusiv_bit_or:
        print("E: reduction(^:exclusiv_bit_or)")
        result = False

    exclusiv_bit_or = 0;
    logics[1000//2]=1

    if 'omp parallel sections private(i) reduction(^:exclusiv_bit_or)':
        if 'omp section':
            for i in range(0, 300):
                exclusiv_bit_or = (exclusiv_bit_or ^ logics[i])
        if 'omp section':
            for i in range(300, 700):
                exclusiv_bit_or = (exclusiv_bit_or ^ logics[i])
        if 'omp section':
            for i in range(700, 1000):
                exclusiv_bit_or = (exclusiv_bit_or ^ logics[i])

    if not exclusiv_bit_or:
        print("E: reduction(^:exclusiv_bit_or) with logics[1000/2]=1")
        result = False
    return result



import math

#pythran export _cross_product (float, float, float, float, float, float)
def _cross_product(x0, y0, x1, y1, x2, y2):
        return (x1 - x0) * (y2 - y0) - (y1 - y0) * (x2 - x0)

#pythran export make_circumcircle ((float, float), (float, float), (float, float))
def make_circumcircle(a, b, c):
    # Mathematical algorithm from Wikipedia: Circumscribed circle
        ox = (min(a[0], b[0], c[0]) + max(a[0], b[0], c[0])) / 2.0
        oy = (min(a[1], b[1], c[1]) + max(a[1], b[1], c[1])) / 2.0
        ax = a[0] - ox;  ay = a[1] - oy
        bx = b[0] - ox;  by = b[1] - oy
        cx = c[0] - ox;  cy = c[1] - oy
        d = (ax * (by - cy) + bx * (cy - ay) + cx * (ay - by)) * 2.0
        if d == 0.0:
            return None
        x = ox + ((ax*ax + ay*ay) * (by - cy) + (bx*bx + by*by) * (cy - ay) + (cx*cx + cy*cy) * (ay - by)) / d
        y = oy + ((ax*ax + ay*ay) * (cx - bx) + (bx*bx + by*by) * (ax - cx) + (cx*cx + cy*cy) * (bx - ax)) / d
        ra = math.hypot(x - a[0], y - a[1])
        rb = math.hypot(x - b[0], y - b[1])
        rc = math.hypot(x - c[0], y - c[1])
        return (x, y, max(ra, rb, rc))

#pythran export make_diameter ((float, float), (float, float))    
def make_diameter(a, b):
    cx = (a[0] + b[0]) / 2.0
    cy = (a[1] + b[1]) / 2.0
    r0 = math.hypot(cx - a[0], cy - a[1])
    r1 = math.hypot(cx - b[0], cy - b[1])
    return (cx, cy, max(r0, r1))

#pythran export is_in_circle((float, float, float), (float, float))
def is_in_circle(c, p):
    return math.hypot(p[0] - c[0], p[1] - c[1]) <= c[2] * (1 + 1e-14)

#pythran export _make_circle_one_point((float,float) list, (float, float))
def _make_circle_one_point(points, p):
        c = (p[0], p[1], 0.0)
        for (i, q) in enumerate(points):
                if not is_in_circle(c, q):
                        if c[2] == 0.0:
                                c = make_diameter(p, q)
                        else:
                                c = make_circle_two_points(points[ : i + 1], p, q)
        return c

#pythran export make_circle_two_points((float,float) list, (float, float), (float, float))
#runas make_circle_two_points([(1.,4.), (0., 8.)], (3., 4.), (5., 6.))
def make_circle_two_points(points, p, q):
    circ = make_diameter(p, q)
    left  = None
    right = None
    px, py = p
    qx, qy = q
    
    # For each point not in the two-point circle
    for r in points:
        if is_in_circle(circ, r):
            continue

        # Form a circumcircle and classify it on left or right side
        cross = _cross_product(px, py, qx, qy, r[0], r[1])
        c = make_circumcircle(p, q, r)
        if c is None:
            continue
        elif cross > 0.0 and (left is None or _cross_product(px, py, qx, qy, c[0], c[1]) > _cross_product(px, py, qx, qy, left[0], left[1])):
            left = c
        elif cross < 0.0 and (right is None or _cross_product(px, py, qx, qy, c[0], c[1]) < _cross_product(px, py, qx, qy, right[0], right[1])):
            right = c
    # Select which circle to return
    if left is None and right is None:
            return circ
    elif left is None:
            return right
    elif right is None:
            return left
    else:
            return left if (left[2] <= right[2]) else right




""" Tests for test cases directory. """

# TODO: check http://code.google.com/p/unladen-swallow/wiki/Benchmarks
import os
from distutils.version import LooseVersion
import numpy
import unittest

from pythran.tests import TestFromDir


class TestCases(TestFromDir):

    """ Class to check all tests in the cases directory. """

    path = os.path.join(os.path.dirname(__file__), "cases")


TestCases.populate(TestCases)

if LooseVersion(numpy.__version__) >= '1.20':
    del TestCases.test_train_equalizer_norun0
    del TestCases.test_train_eq_run0
    del TestCases.test_train_eq_run1

# too template intensive for old g++
if os.environ.get('CXX', None) == 'g++-5':
    del TestCases.test_loopy_jacob_run0


if __name__ == '__main__':
    unittest.main()



import numpy as np

#pythran export train_equaliser(complex128[][], int, int, int, float64, complex128[][][], int[], bool, complex128[][], str)
#pythran export train_equaliser(complex64[][], int, int, int, float32, complex64[][][], int[], bool, complex64[][], str)
def train_equaliser(E, TrSyms, Niter, os, mu, wx, modes, adaptive, symbols,  method):
   if method == "cma":
       errorfct = cma_error

   else:
       raise ValueError("Unknown method %s"%method)
   nmodes = E.shape[0]
   ntaps = wx.shape[-1]
   assert symbols.shape[0] == nmodes, "symbols must be at least size of modes"
   assert wx.shape[0] == nmodes, "wx needs to have at least as many dimensions as the maximum mode"
   assert E.shape[1] > TrSyms*os+ntaps, "Field must be longer than the number of training symbols"
   assert modes.max() < nmodes, "Maximum mode number must not be higher than number of modes"
   err = np.zeros((nmodes, TrSyms*Niter), dtype=E.dtype)
   #omp parallel for
   for mode in modes:
       for it in range(Niter):
           for i in range(TrSyms):
               X = E[:, i * os:i * os + ntaps]
               Xest = apply_filter(X,  wx[mode])
               err[mode, it*Niter+i] = errorfct(Xest, symbols[mode], i)
               wx[mode] += mu * np.conj(err[mode, it*Niter+i]) * X
               if adaptive and i > 0:
                   mu = adapt_step(mu, err[mode, it*Niter+i], err[mode, it*Niter+i-1])
   return err, wx, mu

def cma_error(Xest, s1, i):
   d = s1[0].real - abs(Xest)**2
   return d*Xest


def adapt_step(mu, err_p, err):
    if err.real*err_p.real > 0 and err.imag*err_p.imag > 0:
        return mu
    else:
        return mu/(1+mu*(err.real*err.real + err.imag*err.imag))

def apply_filter(E, wx):
    pols = E.shape[0]
    Ntaps = wx.shape[1]
    Xest = E.dtype.type(0)
    for k in range(pols):
        for i in range(Ntaps):
            Xest += E[k, i]*np.conj(wx[k,i])
    return Xest



#pythran export weights(uint8[:,:],float?)
#runas import numpy as np; x = np.arange(10, dtype=np.uint8).reshape(5,2); weights(x)

import numpy as np
def weights(input_data, threshold=0.3):
    n_seq, length = input_data.shape
    weights = np.zeros(n_seq, dtype=np.float32)

    for i in range(n_seq):
        vector = input_data[i, None, :]
        count_matches = np.sum(vector == input_data, axis=1)
        over_threshold = count_matches > (threshold * length)
        total = np.sum(over_threshold)
        weights[i] = 1 / total

    return weights
